#   Copyright 2016 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import mock

from tripleoclient.tests import fakes
from tripleoclient.tests.v1.overcloud_deploy import fakes as deploy_fakes
from tripleoclient.v2 import tripleo_container_image as tcib


IMAGE_YAML = """---
container_images:
  - image_source: "tripleo"
    imagename: "test/keystone:tag"
"""

MOCK_WALK = [
    ("", ["base"], [],),
    ("/base", ["memcached", "openstack"], ["config.yaml", "test.doc"],),
    ("/base/memcached", [], ["memcached.yaml"],),
    ("/base/openstack", ["glance", "keystone", "neutron", "nova"], [],),
    (
        "/base/openstack/glance",
        [],
        ["glance-registry.yaml", "glance-api.yaml"],
    ),
    ("/base/openstack/keystone", [], ["keystone.yaml"],),
    ("/base/openstack/neutron", ["api"], [],),
    ("/base/openstack/neutron/api", [], ["neutron-api.yml"],),
    ("/base/openstack/nova", [], [],),
]


class TestContainerImages(deploy_fakes.TestDeployOvercloud):
    def setUp(self):
        super(TestContainerImages, self).setUp()
        self.app = fakes.FakeApp()
        self.os_walk = mock.patch(
            "os.walk", autospec=True, return_value=iter(MOCK_WALK)
        )
        self.os_walk.start()
        self.addCleanup(self.os_walk.stop)
        self.os_listdir = mock.patch(
            "os.listdir", autospec=True, return_value=["config.yaml"]
        )
        self.os_listdir.start()
        self.addCleanup(self.os_listdir.stop)
        self.run_ansible_playbook = mock.patch(
            "tripleoclient.utils.run_ansible_playbook", autospec=True
        )
        self.run_ansible_playbook.start()
        self.addCleanup(self.run_ansible_playbook.stop)
        self.buildah_build_all = mock.patch(
            "tripleo_common.image.builder.buildah.BuildahBuilder.build_all",
            autospec=True,
        )
        self.mock_buildah = self.buildah_build_all.start()
        self.addCleanup(self.buildah_build_all.stop)
        self.cmd = tcib.Build(self.app, None)

    def _take_action(self, parsed_args):
        self.cmd.image_parents = {"keystone": "base"}
        mock_open = mock.mock_open(read_data=IMAGE_YAML)
        with mock.patch("os.path.isfile", autospec=True) as mock_isfile:
            mock_isfile.return_value = True
            with mock.patch("os.path.isdir", autospec=True) as mock_isdir:
                mock_isdir.return_value = True
                with mock.patch('six.moves.builtins.open', mock_open):
                    with mock.patch(
                        "tripleoclient.v2.tripleo_container_image.Build"
                        ".find_image",
                        autospec=True,
                    ) as mock_find_image:
                        mock_find_image.return_value = {"tcib_option": "data"}
                        self.cmd.take_action(parsed_args)

    def test_find_image(self):
        mock_open = mock.mock_open(read_data='---\ntcib_option: "data"')
        with mock.patch('six.moves.builtins.open', mock_open):
            image = self.cmd.find_image("keystone", "some/path", "base-image")
        self.assertEqual(image, {"tcib_option": "data"})

    def test_build_tree(self):
        image = self.cmd.build_tree("some/path")
        self.assertEqual(
            image,
            [
                {
                    "base": [
                        "memcached",
                        {
                            "openstack": [
                                "glance",
                                "keystone",
                                {"neutron": ["api"]},
                                "nova",
                            ]
                        },
                    ]
                }
            ],
        )

    def test_image_regex(self):
        image = self.cmd.imagename_to_regex("test/centos-binary-keystone:tag")
        self.assertEqual(image, "keystone")
        image = self.cmd.imagename_to_regex("test/rhel-binary-keystone:tag")
        self.assertEqual(image, "keystone")
        image = self.cmd.imagename_to_regex("test/rhel-source-keystone:tag")
        self.assertEqual(image, "keystone")
        image = self.cmd.imagename_to_regex("test/rhel-rdo-keystone:tag")
        self.assertEqual(image, "keystone")
        image = self.cmd.imagename_to_regex("test/rhel-rhos-keystone:tag")
        self.assertEqual(image, "keystone")
        image = self.cmd.imagename_to_regex("test/other-keystone:tag")
        self.assertEqual(image, "other-keystone")

    def test_rectify_excludes(self):
        self.cmd.identified_images = ["keystone", "nova", "glance"]
        excludes = self.cmd.rectify_excludes(images_to_prepare=["glance"])
        self.assertEqual(excludes, ["keystone", "nova"])

    def test_image_build_yaml(self):
        arglist = ["--config-file", "config.yaml"]
        verifylist = [("config_file", "config.yaml")]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self._take_action(parsed_args=parsed_args)

        assert self.mock_buildah.called

    def test_image_build_with_skip_build(self):
        arglist = ["--config-file", "config.yaml", "--skip-build"]
        verifylist = [("config_file", "config.yaml"), ("skip_build", True)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self._take_action(parsed_args=parsed_args)

        assert not self.mock_buildah.called

    def test_image_build_with_push(self):
        arglist = ["--config-file", "config.yaml", "--push"]
        verifylist = [("config_file", "config.yaml"), ("push", True)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self._take_action(parsed_args=parsed_args)

        assert self.mock_buildah.called

    def test_image_build_with_volume(self):
        arglist = ["--config-file", "config.yaml", "--volume", "bind/mount"]
        verifylist = [
            ("config_file", "config.yaml"),
            (
                "volumes",
                [
                    "/etc/yum.repos.d:/etc/distro.repos.d:z",
                    "/etc/pki/rpm-gpg:/etc/pki/rpm-gpg:z",
                    "bind/mount",
                ],
            ),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self._take_action(parsed_args=parsed_args)

        assert self.mock_buildah.called

    def test_image_build_with_exclude(self):
        arglist = ["--exclude", "image1"]
        verifylist = [
            ("excludes", ["image1"]),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self._take_action(parsed_args=parsed_args)

        assert self.mock_buildah.called

    def test_image_build_failure_no_config_file(self):
        arglist = ["--config-file", "not-a-file-config.yaml"]
        verifylist = [
            ("config_file", "not-a-file-config.yaml"),
        ]

        self.check_parser(self.cmd, arglist, verifylist)

    def test_image_build_config_dir(self):
        arglist = ["--config-file", "config.yaml", "--config-path", "/foo"]
        verifylist = [("config_file", "config.yaml"), ("config_path", "/foo")]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self._take_action(parsed_args=parsed_args)
        self.assertEqual(self.cmd.tcib_config_path, '/foo/tcib')

    def test_image_build_failure_no_config_dir(self):
        arglist = ["--config-path", "not-a-path"]
        verifylist = [
            ("config_path", "not-a-path"),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        with mock.patch("os.path.isfile", autospec=True) as mock_isfile:
            mock_isfile.return_value = True
            self.assertRaises(IOError, self.cmd.take_action, parsed_args)

    def test_process_images(self):
        rtn_value = {'yay': 'values'}
        arglist = ["--config-path", "foobar/"]
        verifylist = [
            ("config_path", "foobar/"),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        expected_images = ['foo', 'foobar']
        image_configs = {}
        self.cmd.tcib_config_path = '/foo/tcib'
        with mock.patch("tripleoclient.v2.tripleo_container_image.Build"
                        ".find_image", autospec=True) as mock_find_image:

            mock_find_image.return_value = rtn_value
            cfgs = self.cmd.process_images(expected_images, parsed_args,
                                           image_configs)
            mock_find_image.assert_called_once_with(self.cmd, 'foo',
                                                    '/foo/tcib', 'ubi8')
        self.assertEqual(cfgs, {'foo': rtn_value})


class TestContainerImagesHotfix(deploy_fakes.TestDeployOvercloud):
    def setUp(self):
        super(TestContainerImagesHotfix, self).setUp()
        self.run_ansible_playbook = mock.patch(
            "tripleoclient.utils.run_ansible_playbook", autospec=True
        )
        self.run_ansible_playbook.start()
        self.addCleanup(self.run_ansible_playbook.stop)
        self.cmd = tcib.HotFix(self.app, None)

    def _take_action(self, parsed_args):
        with mock.patch("os.path.isfile", autospec=True) as mock_isfile:
            mock_isfile.return_value = True
            self.cmd.take_action(parsed_args)

    def test_image_hotfix(self):
        arglist = ["--image", "container1", "--rpms-path", "/opt"]
        verifylist = [
            ("images", ["container1"]),
            ("rpms_path", "/opt"),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self._take_action(parsed_args=parsed_args)

    def test_image_hotfix_multi_image(self):
        arglist = [
            "--image",
            "container1",
            "--image",
            "container2",
            "--rpms-path",
            "/opt",
        ]
        verifylist = [
            ("images", ["container1", "container2"]),
            ("rpms_path", "/opt"),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self._take_action(parsed_args=parsed_args)

    def test_image_hotfix_missing_args(self):
        arglist = []
        verifylist = []

        self.assertRaises(
            deploy_fakes.fakes.utils.ParserException,
            self.check_parser,
            self.cmd,
            arglist,
            verifylist,
        )



#   Copyright 2018 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import mock

from osc_lib.tests import utils

# Load the plugin init module for the plugin list and show commands
from tripleoclient import exceptions
from tripleoclient.v1 import tripleo_upgrade


class TestUpgrade(utils.TestCommand):

    def setUp(self):
        super(TestUpgrade, self).setUp()

        # Get the command object to test
        self.cmd = tripleo_upgrade.Upgrade(self.app, None)
        self.cmd.ansible_dir = '/tmp'
        self.ansible_playbook_cmd = "ansible-playbook"

    @mock.patch('tripleoclient.utils.prompt_user_for_confirmation',
                return_value=True)
    @mock.patch('tripleoclient.v1.tripleo_deploy.Deploy.take_action',
                autospec=True)
    def test_take_action(self, mock_deploy, mock_confirm):
        verifylist = [
            ('local_ip', '127.0.0.1'),
            ('templates', '/tmp/thtroot'),
            ('stack', 'undercloud'),
            ('output_dir', '/my'),
        ]

        parsed_args = self.check_parser(self.cmd,
                                        ['--local-ip', '127.0.0.1',
                                         '--templates', '/tmp/thtroot',
                                         '--stack', 'undercloud',
                                         '--output-dir', '/my',
                                         '-e', '/tmp/thtroot/puppet/foo.yaml',
                                         '-e', '/tmp/thtroot//docker/bar.yaml',
                                         '-e', '/tmp/thtroot42/notouch.yaml',
                                         '-e', '~/custom.yaml',
                                         '-e', 'something.yaml',
                                         '-e', '../../../outside.yaml'],
                                        verifylist)

        self.cmd.take_action(parsed_args)
        parsed_args.standalone = True
        parsed_args.upgrade = True
        mock_deploy.assert_called_with(self.cmd, parsed_args)

    @mock.patch('tripleoclient.utils.prompt_user_for_confirmation',
                return_value=True)
    @mock.patch('tripleoclient.v1.tripleo_deploy.Deploy.take_action',
                autospec=True)
    def test_take_action_prompt(self, mock_deploy, mock_confirm):
        parsed_args = self.check_parser(self.cmd,
                                        ['--local-ip', '127.0.0.1',
                                         '--templates', '/tmp/thtroot',
                                         '--stack', 'undercloud',
                                         '--output-dir', '/my',
                                         '-e', '/tmp/thtroot/puppet/foo.yaml',
                                         '-e', '/tmp/thtroot//docker/bar.yaml',
                                         '-e', '/tmp/thtroot42/notouch.yaml',
                                         '-e', '~/custom.yaml',
                                         '-e', 'something.yaml',
                                         '-e', '../../../outside.yaml'], [])
        self.cmd.take_action(parsed_args)
        parsed_args.standlone = True
        parsed_args.upgrade = True
        mock_deploy.assert_called_with(self.cmd, parsed_args)

    @mock.patch('tripleoclient.utils.prompt_user_for_confirmation',
                return_value=False)
    @mock.patch('tripleoclient.v1.tripleo_deploy.Deploy',
                autospec=True)
    def test_take_action_prompt_no(self, mock_deploy, mock_confirm):
        parsed_args = self.check_parser(self.cmd,
                                        ['--local-ip', '127.0.0.1',
                                         '--templates', '/tmp/thtroot',
                                         '--stack', 'undercloud',
                                         '--output-dir', '/my',
                                         '-e', '/tmp/thtroot/puppet/foo.yaml',
                                         '-e', '/tmp/thtroot//docker/bar.yaml',
                                         '-e', '/tmp/thtroot42/notouch.yaml',
                                         '-e', '~/custom.yaml',
                                         '-e', 'something.yaml',
                                         '-e', '../../../outside.yaml'], [])
        parsed_args.standlone = True
        parsed_args.upgrade = True
        self.assertRaises(exceptions.UndercloudUpgradeNotConfirmed,
                          self.cmd.take_action, parsed_args)
        mock_deploy.assert_not_called()



#   Copyright 2015 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
from oslo_config import cfg
from oslo_log import log as logging

from osc_lib.i18n import _
from osc_lib import utils

from tripleoclient.exceptions import OvercloudUpdateNotConfirmed

from tripleoclient import command
from tripleoclient import constants
from tripleoclient import utils as oooutils
from tripleoclient.v1.overcloud_deploy import DeployOvercloud
from tripleoclient.workflows import deployment


CONF = cfg.CONF


class UpdatePrepare(DeployOvercloud):
    """Run heat stack update for overcloud nodes to refresh heat stack outputs.

       The heat stack outputs are what we use later on to generate ansible
       playbooks which deliver the minor update workflow. This is used as the
       first step for a minor update of your overcloud.
    """

    log = logging.getLogger(__name__ + ".MinorUpdatePrepare")

    def get_parser(self, prog_name):
        parser = super(UpdatePrepare, self).get_parser(prog_name)
        parser.add_argument('-y', '--yes', default=False,
                            action='store_true',
                            help=_("Use -y or --yes to skip the confirmation "
                                   "required before any update operation. "
                                   "Use this with caution! "),
                            )
        return parser

    def take_action(self, parsed_args):
        logging.register_options(CONF)
        logging.setup(CONF, '')
        self.log.debug("take_action(%s)" % parsed_args)
        oooutils.ensure_run_as_normal_user()

        if (not parsed_args.yes
                and not oooutils.prompt_user_for_confirmation(
                    constants.UPDATE_PROMPT, self.log)):
            raise OvercloudUpdateNotConfirmed(constants.UPDATE_NO)

        # In case of update and upgrade we need to force the
        # config_download to false. The heat stack update will be performed
        # by DeployOvercloud class but skipping the config download part.
        parsed_args.config_download = False

        # Add the update-prepare.yaml environment to set noops etc
        templates_dir = (parsed_args.templates or
                         constants.TRIPLEO_HEAT_TEMPLATES)
        parsed_args.environment_files = oooutils.prepend_environment(
            parsed_args.environment_files, templates_dir,
            constants.UPDATE_PREPARE_ENV)

        # Throw deprecation warning if service is enabled and
        # ask user if update should still be continued.
        if parsed_args.environment_files:
            oooutils.check_deprecated_service_is_enabled(
                parsed_args.environment_files)

        super(UpdatePrepare, self).take_action(parsed_args)
        self.log.info("Update init on stack {0} complete.".format(
                      parsed_args.stack))


class UpdateRun(command.Command):
    """Run minor update ansible playbooks on Overcloud nodes"""

    log = logging.getLogger(__name__ + ".MinorUpdateRun")

    def get_parser(self, prog_name):
        parser = super(UpdateRun, self).get_parser(prog_name)
        parser.add_argument(
            '--limit',
            action='store',
            required=True,
            help=_("A string that identifies a single node or comma-separated"
                   "list of nodes the config-download Ansible playbook "
                   "execution will be limited to. For example: --limit"
                   " \"compute-0,compute-1,compute-5\".")
        )
        parser.add_argument('--playbook',
                            nargs="*",
                            default=None,
                            help=_("Ansible playbook to use for the minor"
                                   " update. Can be used multiple times."
                                   " Set this to each of those playbooks in"
                                   " consecutive invocations of this command"
                                   " if you prefer to run them manually."
                                   " Note: make sure to run all playbooks so"
                                   " that all services are updated and running"
                                   " with the target version configuration.")
                            )
        parser.add_argument("--ssh-user",
                            dest="ssh_user",
                            action="store",
                            default="tripleo-admin",
                            help=_("DEPRECATED: Only tripleo-admin should be "
                                   "used as ssh user.")
                            )
        parser.add_argument('--static-inventory',
                            dest='static_inventory',
                            action="store",
                            default=None,
                            help=_('Path to an existing ansible inventory to '
                                   'use. If not specified, one will be '
                                   'generated in '
                                   '~/tripleo-ansible-inventory.yaml')
                            )
        parser.add_argument('--stack', dest='stack',
                            help=_('Name or ID of heat stack '
                                   '(default=Env: OVERCLOUD_STACK_NAME)'),
                            default=utils.env('OVERCLOUD_STACK_NAME',
                                              default='overcloud')
                            )
        parser.add_argument('--no-workflow', dest='no_workflow',
                            action='store_true',
                            default=True,
                            help=_('This option no longer has any effect.')
                            )
        parser.add_argument(
            '--tags',
            action='store',
            default=None,
            help=_('A list of tags to use when running the the config-download'
                   ' ansible-playbook command.')
        )
        parser.add_argument(
            '--skip-tags',
            action='store',
            default=None,
            help=_('A list of tags to skip when running the the'
                   ' config-download ansible-playbook command.')
        )
        parser.add_argument(
            '-y', '--yes',
            default=False,
            action='store_true',
            help=_("Use -y or --yes to skip the confirmation required before "
                   "any update operation. Use this with caution! "),
        )
        parser.add_argument(
            '--ansible-forks',
            action='store',
            default=None,
            type=int,
            help=_('The number of Ansible forks to use for the'
                   ' config-download ansible-playbook command.')
        )
        return parser

    def take_action(self, parsed_args):
        logging.register_options(CONF)
        logging.setup(CONF, '')
        self.log.debug("take_action(%s)" % parsed_args)
        oooutils.ensure_run_as_normal_user()

        if (not parsed_args.yes
            and not oooutils.prompt_user_for_confirmation(
                    constants.UPDATE_PROMPT, self.log)):
            raise OvercloudUpdateNotConfirmed(constants.UPDATE_NO)
        # NOTE(cloudnull): The string option "all" was a special default
        #                  that is no longer relevant. To retain compatibility
        #                  this condition has been put in place.
        if not parsed_args.playbook or parsed_args.playbook == ['all']:
            playbook = constants.MINOR_UPDATE_PLAYBOOKS
        else:
            playbook = parsed_args.playbook

        _, ansible_dir = self.get_ansible_key_and_dir(
            no_workflow=True,
            stack=parsed_args.stack,
            orchestration=self.app.client_manager.orchestration
        )
        deployment.config_download(
            log=self.log,
            clients=self.app.client_manager,
            stack=oooutils.get_stack(
                self.app.client_manager.orchestration,
                parsed_args.stack
            ),
            output_dir=ansible_dir,
            verbosity=oooutils.playbook_verbosity(self=self),
            ansible_playbook_name=playbook,
            inventory_path=oooutils.get_tripleo_ansible_inventory(
                parsed_args.static_inventory,
                parsed_args.ssh_user,
                parsed_args.stack,
                return_inventory_file_path=True
            ),
            limit_hosts=oooutils.playbook_limit_parse(
                limit_nodes=parsed_args.limit
            ),
            skip_tags=parsed_args.skip_tags,
            tags=parsed_args.tags,
            forks=parsed_args.ansible_forks
        )
        self.log.info("Completed Overcloud Minor Update Run.")


class UpdateConverge(DeployOvercloud):
    """Converge the update on Overcloud nodes.

    This restores the plan and stack so that normal deployment
    workflow is back in place.
    """

    log = logging.getLogger(__name__ + ".UpdateConverge")

    def get_parser(self, prog_name):
        parser = super(UpdateConverge, self).get_parser(prog_name)
        parser.add_argument('-y', '--yes', default=False,
                            action='store_true',
                            help=_("Use -y or --yes to skip the confirmation "
                                   "required before any update operation. "
                                   "Use this with caution! "),
                            )
        return parser

    def take_action(self, parsed_args):
        logging.register_options(CONF)
        logging.setup(CONF, '')
        self.log.debug("take_action(%s)" % parsed_args)
        oooutils.ensure_run_as_normal_user()

        if (not parsed_args.yes
            and not oooutils.prompt_user_for_confirmation(
                    constants.UPDATE_PROMPT, self.log)):
            raise OvercloudUpdateNotConfirmed(constants.UPDATE_NO)

        # Add the update-converge.yaml environment to unset noops
        templates_dir = (parsed_args.templates or
                         constants.TRIPLEO_HEAT_TEMPLATES)
        parsed_args.environment_files = oooutils.prepend_environment(
            parsed_args.environment_files, templates_dir,
            constants.UPDATE_CONVERGE_ENV)

        super(UpdateConverge, self).take_action(parsed_args)
        self.log.info("Update converge on stack {0} complete.".format(
                      parsed_args.stack))



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import mock

from tripleoclient.tests.v1 import test_plugin
from tripleoclient import utils
from tripleoclient.v1 import overcloud_admin
from tripleoclient.workflows import deployment


@mock.patch.object(utils, 'get_stack', autospec=True)
@mock.patch.object(deployment, 'get_hosts_and_enable_ssh_admin', autospec=True)
class TestAdminAuthorize(test_plugin.TestPluginV1):
    def setUp(self):
        super(TestAdminAuthorize, self).setUp()
        self.cmd = overcloud_admin.Authorize(self.app, None)
        self.app.client_manager = mock.Mock()

    def test_ok(self, mock_get_host_and_enable_ssh_admin, mock_get_stack):
        arglist = []
        parsed_args = self.check_parser(self.cmd, arglist, [])
        mock_stack = mock.Mock()
        mock_get_stack.return_value = mock_stack

        self.cmd.take_action(parsed_args)
        mock_get_host_and_enable_ssh_admin.assert_called_once_with(
            mock_stack,
            parsed_args.overcloud_ssh_network,
            parsed_args.overcloud_ssh_user,
            mock.ANY,
            parsed_args.overcloud_ssh_port_timeout,
            mock.ANY
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging
import os
import re
import yaml

from tripleo_common.utils import stack_parameters as stk_parameters

from tripleoclient.constants import ANSIBLE_TRIPLEO_PLAYBOOKS
from tripleoclient.constants import UNUSED_PARAMETER_EXCLUDES_RE
from tripleoclient import exceptions
from tripleoclient import utils
from tripleoclient.workflows import roles


LOG = logging.getLogger(__name__)


def invoke_plan_env_workflows(clients, stack_name, plan_env_file,
                              verbosity=0):
    """Invokes the workflows in plan environment file"""

    try:
        with open(plan_env_file) as pf:
            plan_env_data = yaml.safe_load(pf.read())
    except IOError as exc:
        raise exceptions.PlanEnvWorkflowError('File (%s) is not found: '
                                              '%s' % (plan_env_file, exc))

    static_inventory = utils.get_tripleo_ansible_inventory(
        ssh_user='heat-admin',
        stack=stack_name,
        undercloud_connection='local',
        return_inventory_file_path=True
    )
    with utils.TempDirs() as tmp:
        for pb, pb_vars in plan_env_data["playbook_parameters"].items():
            print(
                'Invoking playbook ({}) specified in'
                ' plan-environment file'.format(pb)
            )
            LOG.debug(
                'Running playbook "{}" with the'
                ' following options {}.'.format(
                    pb,
                    pb_vars
                )
            )
            playbook_dir = os.path.dirname(pb)
            if not playbook_dir:
                playbook_dir = ANSIBLE_TRIPLEO_PLAYBOOKS

            utils.run_ansible_playbook(
                playbook=os.path.basename(pb),
                inventory=static_inventory,
                workdir=tmp,
                playbook_dir=playbook_dir,
                verbosity=verbosity,
                extra_vars=pb_vars
            )


def check_deprecated_parameters(clients, container):
    """Checks for deprecated parameters in plan and adds warning if present.

    :param clients: application client object.
    :type clients: Object

    :param container: Name of the stack container.
    :type container: String
    """
    role_name_list = roles.list_available_roles(
        clients=clients,
        container=container
    )
    flattened_parms = stk_parameters.get_flattened_parameters(
        clients.tripleoclient.object_store,
        clients.orchestration, container=container)
    user_params = flattened_parms.get('environment_parameters', {})
    heat_resource_tree = flattened_parms.get('heat_resource_tree', {})
    heat_resource_tree_params = heat_resource_tree.get('parameters', {})
    heat_resource_tree_resources = heat_resource_tree.get('resources', {})
    plan_params = heat_resource_tree_params.keys()
    parameter_groups = [
        i.get('parameter_groups')
        for i in heat_resource_tree_resources.values()
        if i.get('parameter_groups')
    ]
    params_role_specific_tag = [
        i.get('name')
        for i in heat_resource_tree_params.values()
        if 'tags' in i and 'role_specific' in i['tags']
    ]

    r = re.compile(".*Count")
    filtered_names = list(filter(r.match, plan_params))
    valid_role_name_list = list()
    for name in filtered_names:
        default = heat_resource_tree_params[name].get('default', 0)
        if default and int(default) > 0:
            role_name = name.rstrip('Count')
            if [i for i in role_name_list if i.get('name') == role_name]:
                valid_role_name_list.append(role_name)

    deprecated_params = [
        i[0] for i in parameter_groups
        if i[0].get('label') == 'deprecated'
    ]
    # We are setting a frozenset here because python 3 complains that dict is
    # a unhashable type.
    # On user_defined, we check if the size is higher than 0 because an empty
    # frozenset still is a subset of a frozenset, so we can't use issubset
    # here.
    user_params_keys = frozenset(user_params.keys())
    deprecated_result = [
        {
            'parameter': i,
            'deprecated': True,
            'user_defined': len(
                [x for x in frozenset(i) if x in user_params_keys]) > 0
        }
        for i in deprecated_params
    ]
    unused_params = [i for i in user_params.keys() if i not in plan_params]
    user_provided_role_specific = [
        v for i in role_name_list
        for k, v in user_params.items()
        if k in i
    ]
    invalid_role_specific_params = [
        i for i in user_provided_role_specific
        if i in params_role_specific_tag
    ]
    deprecated_parameters = [
        param['parameter'] for param in deprecated_result
        if param.get('user_defined')
    ]

    if deprecated_parameters:
        deprecated_join = ', '.join(deprecated_parameters)
        LOG.warning(
            'WARNING: Following parameter(s) are deprecated and still '
            'defined. Deprecated parameters will be removed soon!'
            ' {deprecated_join}'.format(
                deprecated_join=deprecated_join
            )
        )

    # exclude our known params that may not be used
    ignore_re = re.compile('|'.join(UNUSED_PARAMETER_EXCLUDES_RE))
    unused_params = [p for p in unused_params if not ignore_re.search(p)]

    if unused_params:
        unused_join = ', '.join(unused_params)
        LOG.warning(
            'WARNING: Following parameter(s) are defined but not '
            'currently used in the deployment plan. These parameters '
            'may be valid but not in use due to the service or '
            'deployment configuration.'
            ' {unused_join}'.format(
                unused_join=unused_join
            )
        )

    if invalid_role_specific_params:
        invalid_join = ', '.join(invalid_role_specific_params)
        LOG.warning(
            'WARNING: Following parameter(s) are not supported as '
            'role-specific inputs. {invalid_join}'.format(
                invalid_join=invalid_join
            )
        )


def generate_fencing_parameters(clients, nodes_json, delay, ipmi_level,
                                ipmi_cipher, ipmi_lanplus):
    """Generate and return fencing parameters.

    :param clients: application client object.
    :type clients: Object

    :param nodes_json: list of nodes & attributes in json format
    :type nodes_json: List

    :param delay: time to wait before taking fencing action
    :type delay: Integer

    :param ipmi_level: IPMI user level to use
    :type ipmi_level: String

    :param ipmi_cipher: IPMI cipher suite to use
    :type ipmi_cipher: String

    :param ipmi_lanplus: whether to use IPMIv2.0
    :type ipmi_lanplus: Boolean

    :returns: Dictionary
    """
    return stk_parameters.generate_fencing_parameters(
        clients.baremetal,
        clients.compute,
        nodes_json=nodes_json,
        delay=delay,
        ipmi_level=ipmi_level,
        ipmi_cipher=ipmi_cipher,
        ipmi_lanplus=ipmi_lanplus)


def check_forbidden_params(log, env_files, forbidden):
    """Looks for undesired parameters in the environment files.

    Each of the environment files pass in env_files will be parsed
    and if the parameters_default key is found, then all the keys
    from the nested dictionary found under will be converted into
    a list, for example:

    parameters_default:
        key1: value1
        key2: value2
        key3:
          - value3
          - key31:
              key311: value311
              key312: value312
            key32: value32

    Will be converted by get_all_keys into:
    [key1, key2, key3, key31, key311, key312, key32]

    This list provides us with all the parameters used in the environment
    file, without the values, in the format of a list. So we can use sets
    to find occurrences of the forbbiden paramenters.

    The variable matched_params will get all the ocurrences of forbidden
    parameters stored, so we can parse all the environment files and show
    all the parameters which should get removed from the environment files
    at once (saving the user to run the command, modify a template, run it
    again, modify another, etc...). If matched_params list is not empty,
    an exception will be raised, stopping the execution of the command and
    displaying the commands which need to be removed.

    :param log: logging object passed from the calling method
    :type log: Logging object
    :param env_files: list of the environment files passed in the command
    :type env_files: list of strings
    :param forbidden: list of the undesired parameters
    :type forbidden: list of strings

    :returns exception if some of the forbidden parameters are found in
    the environment files.
    """

    # Iterates over a nested dict and returns all the
    # keys from the dict in a list
    # example:
    #   * input: {'a': '1', 'b': ['c': '2', 'd': {'e': '3'}]}
    #   * output: ['a', 'b', 'c', 'd', 'e']
    def get_all_keys(obj, keys_list):
        if isinstance(obj, dict):
            keys_list += obj.keys()
            for value in obj.values():
                get_all_keys(value, keys_list)
        elif isinstance(obj, list):
            for value in obj:
                get_all_keys(value, keys_list)

    matched_params = []

    for file in env_files:
        if os.path.exists(file):
            with open(file, 'r') as env_file:
                contents = yaml.safe_load(env_file)
                pd = contents.get('parameter_defaults', {})
                if pd:
                    # Intersection of values and forbidden params
                    list_of_keys = []
                    get_all_keys(pd, list_of_keys)
                    found_in_pd = list(set(list_of_keys) & set(forbidden))

                    # Combine them without duplicates
                    matched_params = list(set(matched_params + found_in_pd))

    if matched_params:
        raise exceptions.BannedParameters("The following parameters should be "
                                          "removed from the environment files:"
                                          "\n{}\n"
                                          .format('\n'.join(matched_params)))



# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock

from osc_lib.tests import utils

from tripleoclient import constants
from tripleoclient.tests import base
from tripleoclient.workflows import plan_management


class TestPlanCreationWorkflows(utils.TestCommand):

    def setUp(self):
        super(TestPlanCreationWorkflows, self).setUp()
        self.tripleoclient = mock.Mock()
        self.app.client_manager.tripleoclient = self.tripleoclient
        self.tripleoclient.object_store.get_account = mock.MagicMock()
        self.mock_tar = mock.patch(
            'tripleo_common.utils.tarball.create_tarball',
            autospec=True
        )
        self.mock_tar.start()

    def tearDown(self):
        super(TestPlanCreationWorkflows, self).tearDown()
        self.mock_tar.stop()

    @mock.patch("tripleoclient.utils.run_ansible_playbook", autospec=True)
    @mock.patch('os.chdir', autospec=True)
    @mock.patch('tempfile.mkdtemp', autospec=True)
    def test_create_plan_from_templates_success(self, mock_tmp, mock_cd,
                                                mock_run_playbook):
        plan_management.create_plan_from_templates(
            self.app.client_manager,
            'test-overcloud',
            '/tht-root/')

        mock_run_playbook.assert_called_with(
            'cli-create-deployment-plan.yaml',
            'undercloud,',
            mock.ANY,
            constants.ANSIBLE_TRIPLEO_PLAYBOOKS,
            extra_vars={
                "container": "test-overcloud",
                "generate_passwords": True,
                "use_default_templates": False,
                "disable_image_params_prepare": False,
            },
            verbosity=0,
        )

    @mock.patch("tripleoclient.utils.run_ansible_playbook", autospec=True)
    @mock.patch('tripleoclient.utils.rel_or_abs_path')
    @mock.patch('os.chdir', autospec=True)
    @mock.patch('tempfile.mkdtemp', autospec=True)
    def test_create_plan_from_templates_roles_data(self, mock_tmp, mock_cd,
                                                   mock_norm_path,
                                                   mock_run_playbook):
        mock_open_context = mock.mock_open()
        with mock.patch('six.moves.builtins.open', mock_open_context):
            plan_management.create_plan_from_templates(
                self.app.client_manager,
                'test-overcloud',
                '/tht-root/',
                'the_roles_file.yaml')

        mock_run_playbook.assert_called_with(
            'cli-create-deployment-plan.yaml',
            'undercloud,',
            mock.ANY,
            constants.ANSIBLE_TRIPLEO_PLAYBOOKS,
            extra_vars={
                "container": "test-overcloud",
                "generate_passwords": True,
                "use_default_templates": False,
                "disable_image_params_prepare": False,
            },
            verbosity=0,
        )

        self.assertIn(mock.call('the_roles_file.yaml', '/tht-root/'),
                      mock_norm_path.call_args_list)

        self.tripleoclient.object_store.put_object.assert_called_with(
            'test-overcloud', 'roles_data.yaml', mock_open_context())

    @mock.patch("tripleoclient.utils.run_ansible_playbook", autospec=True)
    @mock.patch('os.chdir', autospec=True)
    @mock.patch('tempfile.mkdtemp', autospec=True)
    def test_create_plan_from_templates_plan_env_data(self, mock_tmp, mock_cd,
                                                      mock_run_playbook):
        mock_open_context = mock.mock_open()
        with mock.patch('six.moves.builtins.open', mock_open_context):
            plan_management.create_plan_from_templates(
                self.app.client_manager,
                'test-overcloud',
                '/tht-root/',
                plan_env_file='the-plan-environment.yaml')

        mock_run_playbook.assert_called_with(
            'cli-create-deployment-plan.yaml',
            'undercloud,',
            mock.ANY,
            constants.ANSIBLE_TRIPLEO_PLAYBOOKS,
            extra_vars={
                "container": "test-overcloud",
                "generate_passwords": True,
                "plan_environment": "the-plan-environment.yaml",
                "use_default_templates": False,
                "disable_image_params_prepare": False,
            },
            verbosity=0,
        )
        mock_open_context.assert_has_calls(
            [mock.call('the-plan-environment.yaml', 'rb')])

        self.tripleoclient.object_store.put_object.assert_called_with(
            'test-overcloud', 'plan-environment.yaml', mock_open_context())

    @mock.patch("tripleoclient.utils.run_ansible_playbook", autospec=True)
    @mock.patch('os.chdir', autospec=True)
    @mock.patch('tempfile.mkdtemp', autospec=True)
    def test_create_plan_from_templates_networks_data(self, mock_tmp, mock_cd,
                                                      mock_run_playbook):
        mock_open_context = mock.mock_open()
        with mock.patch('six.moves.builtins.open', mock_open_context):
            plan_management.create_plan_from_templates(
                self.app.client_manager,
                'test-overcloud',
                '/tht-root/',
                networks_file='the-network-data.yaml')

        mock_run_playbook.assert_called_with(
            'cli-create-deployment-plan.yaml',
            'undercloud,',
            mock.ANY,
            constants.ANSIBLE_TRIPLEO_PLAYBOOKS,
            extra_vars={
                "container": "test-overcloud",
                "generate_passwords": True,
                "use_default_templates": False,
                "disable_image_params_prepare": False,
            },
            verbosity=0,
        )
        mock_open_context.assert_has_calls(
            [mock.call('the-network-data.yaml', 'rb')])

        self.tripleoclient.object_store.put_object.assert_called_with(
            'test-overcloud', 'network_data.yaml', mock_open_context())

    @mock.patch("tripleoclient.utils.run_ansible_playbook", autospec=True)
    @mock.patch('os.chdir', autospec=True)
    @mock.patch('tempfile.mkdtemp', autospec=True)
    def test_create_plan_with_password_gen_disabled(self, mock_tmp, mock_cd,
                                                    mock_run_playbook):
        plan_management.create_plan_from_templates(
            self.app.client_manager,
            'test-overcloud',
            '/tht-root/',
            generate_passwords=False,
            disable_image_params_prepare=True)

        mock_run_playbook.assert_called_with(
            'cli-create-deployment-plan.yaml',
            'undercloud,',
            mock.ANY,
            constants.ANSIBLE_TRIPLEO_PLAYBOOKS,
            extra_vars={
                "container": "test-overcloud",
                "generate_passwords": False,
                "use_default_templates": False,
                "disable_image_params_prepare": True,
            },
            verbosity=0,
        )


class TestPlanUpdateWorkflows(base.TestCommand):

    def setUp(self):
        super(TestPlanUpdateWorkflows, self).setUp()
        self.app.client_manager.tripleoclient = self.tripleoclient = \
            mock.Mock()
        self.tripleoclient.object_store = self.object_store = mock.Mock()

        self.object_store.get_container.return_value = (
            {},
            [
                {'name': 'plan-environment.yaml'},
                {'name': 'user-environment.yaml'},
                {'name': 'roles_data.yaml'},
                {'name': 'network_data.yaml'},
                {'name': 'user-files/somecustomfile.yaml'},
                {'name': 'user-files/othercustomfile.yaml'},
                {'name': 'this-should-not-be-persisted.yaml'},
            ]
        )

        def get_object(*args, **kwargs):
            if args[0] != 'test-overcloud':
                raise RuntimeError('Unexpected container')
            if args[1] == 'plan-environment.yaml':
                return {}, ('passwords: somepasswords\n'
                            'plan-environment.yaml: mock content\n')
            # Generic return valuebased on param,
            # e.g. 'plan-environment.yaml: mock content'
            return {}, '{0}: mock content\n'.format(args[1])
        self.object_store.get_object.side_effect = get_object
        self.mock_tar = mock.patch(
            'tripleo_common.utils.tarball.create_tarball',
            autospec=True
        )
        self.mock_tar.start()

    def tearDown(self):
        super(TestPlanUpdateWorkflows, self).tearDown()
        self.mock_tar.stop()

    @mock.patch("tripleoclient.utils.run_ansible_playbook", autospec=True)
    @mock.patch('tripleo_common.utils.swift.empty_container',
                autospec=True)
    @mock.patch('os.chdir', autospec=True)
    @mock.patch('tempfile.mkdtemp', autospec=True)
    def test_update_plan_from_templates_keep_env(
            self, mock_tmp, mock_cd, mock_empty_container,
            mock_run_playbook):

        plan_management.update_plan_from_templates(
            self.app.client_manager,
            'test-overcloud',
            '/tht-root/',
            keep_env=True)

        mock_empty_container.assert_called_once_with(
            self.object_store, 'test-overcloud')

        # make sure we're pushing the saved files back to plan
        self.object_store.put_object.assert_has_calls(
            [
                mock.call('test-overcloud', 'plan-environment.yaml',
                          'passwords: somepasswords\n'
                          'plan-environment.yaml: mock content\n'),
                mock.call('test-overcloud', 'user-environment.yaml',
                          'user-environment.yaml: mock content\n'),
                mock.call('test-overcloud', 'roles_data.yaml',
                          'roles_data.yaml: mock content\n'),
                mock.call('test-overcloud', 'network_data.yaml',
                          'network_data.yaml: mock content\n'),
                mock.call('test-overcloud', 'user-files/somecustomfile.yaml',
                          'user-files/somecustomfile.yaml: mock content\n'),
                mock.call('test-overcloud', 'user-files/othercustomfile.yaml',
                          'user-files/othercustomfile.yaml: mock content\n'),
            ],
            any_order=True,
        )
        mock_run_playbook.assert_called_with(
            'cli-update-deployment-plan.yaml',
            'undercloud,',
            mock.ANY,
            constants.ANSIBLE_TRIPLEO_PLAYBOOKS,
            extra_vars={
                "container": "test-overcloud",
                "generate_passwords": True,
                "disable_image_params_prepare": False,
            },
            verbosity=1,
        )

    @mock.patch("tripleoclient.utils.run_ansible_playbook", autospec=True)
    @mock.patch('tripleo_common.utils.swift.empty_container',
                autospec=True)
    def test_update_plan_from_templates_recreate_env(
            self, mock_empty_container, mock_run_playbook):

        plan_management.update_plan_from_templates(
            self.app.client_manager,
            'test-overcloud',
            '/tht-root/')

        mock_empty_container.assert_called_once_with(
            self.object_store, 'test-overcloud')

        # make sure passwords got persisted
        self.object_store.put_object.assert_called_with(
            'test-overcloud', 'plan-environment.yaml',
            'passwords: somepasswords\n'
            'plan-environment.yaml: mock content\n'
        )

        mock_run_playbook.assert_called_with(
            'cli-update-deployment-plan.yaml',
            'undercloud,',
            mock.ANY,
            constants.ANSIBLE_TRIPLEO_PLAYBOOKS,
            extra_vars={
                "container": "test-overcloud",
                "generate_passwords": True,
                "disable_image_params_prepare": False,
            },
            verbosity=1,
        )

    @mock.patch("tripleoclient.utils.run_ansible_playbook", autospec=True)
    @mock.patch('tripleoclient.workflows.plan_management._update_passwords',
                autospec=True)
    @mock.patch('yaml.safe_load',
                autospec=True)
    @mock.patch('tripleo_common.utils.swift.empty_container',
                autospec=True)
    @mock.patch('os.chdir', autospec=True)
    @mock.patch('tempfile.mkdtemp', autospec=True)
    def test_update_plan_from_templates_recreate_env_missing_passwords(
            self, mock_tmp, mock_cd, mock_empty_container,
            mock_yaml_safe_load, mock_update_passwords, mock_run_playbook):
        plan_management.update_plan_from_templates(
            self.app.client_manager,
            'test-overcloud',
            '/tht-root/',
            disable_image_params_prepare=True)
        # A dictionary without the "passwords" key is provided in
        # the _load_passwords method.
        mock_yaml_safe_load.return_value = {}
        # Ensure that the passwords variable is passed with a value of None.
        mock_update_passwords.assert_called_with(
            mock.ANY, 'test-overcloud', None)
        mock_run_playbook.assert_called_with(
            'cli-update-deployment-plan.yaml',
            'undercloud,',
            mock.ANY,
            constants.ANSIBLE_TRIPLEO_PLAYBOOKS,
            extra_vars={
                "container": "test-overcloud",
                "generate_passwords": True,
                "disable_image_params_prepare": True,
            },
            verbosity=1,
        )


class TestUpdatePasswords(base.TestCase):

    YAML_CONTENTS = """version: 1.0
name: overcloud
template: overcloud.yaml
parameter_defaults:
  ControllerCount: 7
"""

    def setUp(self):
        super(TestUpdatePasswords, self).setUp()
        self.swift_client = mock.MagicMock()
        self.swift_client.get_object.return_value = ({}, self.YAML_CONTENTS)

        self.plan_name = "overcast"

    def test_update_passwords(self):
        plan_management._update_passwords(self.swift_client,
                                          self.plan_name,
                                          {'AdminPassword': "1234"})

        result = self.swift_client.put_object.call_args_list[0][0][2]

        # Check new data is in
        self.assertIn("passwords:\n", result)
        self.assertIn("\n  AdminPassword: '1234'", result)
        # Check previous data still is too
        self.assertIn("name: overcloud", result)

    def test_no_passwords(self):
        plan_management._update_passwords(self.swift_client,
                                          self.plan_name,
                                          [])

        self.swift_client.put_object.assert_not_called()

    def test_no_plan_environment(self):
        self.swift_client.get_object.side_effect = (
            Exception("404"))

        plan_management._update_passwords(self.swift_client,
                                          self.plan_name,
                                          {'SecretPassword': 'abcd'})

        self.swift_client.put_object.assert_not_called()



# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six

from tripleo_common.actions import baremetal

from tripleoclient import constants
from tripleoclient import exceptions
from tripleoclient import utils


def validate_nodes(clients, nodes_json):
    """Validate nodes.

    :param clients: Application client object.
    :type clients: Object

    :param nodes_json:
    :type nodes_json: Object

    :returns: Boolean
    """

    context = clients.tripleoclient.create_mistral_context()
    nodes = baremetal.ValidateNodes(nodes_json=nodes_json)
    validated_nodes = nodes.run(context=context)
    if not validated_nodes:
        return True
    else:
        raise exceptions.RegisterOrUpdateError(validated_nodes)


def register_or_update(clients, nodes_json, kernel_name=None,
                       ramdisk_name=None, instance_boot_option=None):
    """Node Registration or Update

    :param clients: Application client object.
    :type clients: Object

    :param nodes_json:
    :type nodes_json: Object

    :param kernel_name: Kernel to use
    :type kernel_name: String

    :param ramdisk_name: RAMDISK to use
    :type ramdisk_name: String

    :param instance_boot_option: Whether to set instances for booting from
                                 local hard drive (local) or network
                                 (netboot).
    :type instance_boot_option: String

    :returns: List
    """

    context = clients.tripleoclient.create_mistral_context()
    nodes = baremetal.RegisterOrUpdateNodes(
        nodes_json=nodes_json,
        ramdisk_name=ramdisk_name,
        kernel_name=kernel_name,
        instance_boot_option=instance_boot_option
    )

    registered_nodes = nodes.run(context=context)
    if not isinstance(registered_nodes, list):
        raise exceptions.RegisterOrUpdateError(registered_nodes)
    else:
        for node in registered_nodes:
            if node.provision_state == 'enroll':
                clients.baremetal.node.set_provision_state(
                    node_uuid=node.uuid,
                    state='manage'
                )
                print('Successfully registered node UUID {}'.format(node.uuid))
            else:
                print('Node UUID {} is already registered'.format(node.uuid))

    return registered_nodes


def _format_errors(payload):
    errors = []
    messages = payload.get('message', [])
    for msg in messages:
        # Adapt for different formats
        if isinstance(msg, six.string_types):
            text = msg
        else:
            text = msg.get('result') or msg.get('message', '')
        try:
            # With multiple workflows, the error message can become
            # quite large and unreadable as it gets passed from task to
            # task. This attempts to keep only the last, and hopefully
            # useful part.
            errors.append(text.rstrip('\n').split('\n')[-1])
        except Exception:
            errors.append(text)
    return '\n'.join(errors)


def provide(verbosity, node_uuids):
    """Provide Baremetal Nodes

    :param verbosity: Verbosity level
    :type verbosity: Integer

    :param node_uuids: List of instance UUID(s).
    :type node_uuids: List
    """

    with utils.TempDirs() as tmp:
        utils.run_ansible_playbook(
            playbook='cli-overcloud-node-provide.yaml',
            inventory='localhost,',
            workdir=tmp,
            playbook_dir=constants.ANSIBLE_TRIPLEO_PLAYBOOKS,
            verbosity=verbosity,
            extra_vars={
                'node_uuids': node_uuids
            }
        )

    print('Successfully provided nodes: {}'.format(node_uuids))


def provide_manageable_nodes(clients, verbosity=0):
    """Provide all manageable Nodes

    :param clients: Application client object.
    :type clients: Object

    :param verbosity: Verbosity level
    :type verbosity: Integer
    """

    provide(
        verbosity=verbosity,
        node_uuids=[
            i.uuid for i in clients.baremetal.node.list()
            if i.provision_state == "manageable" and not i.maintenance
        ]
    )


def introspect(clients, node_uuids, run_validations, concurrency,
               node_timeout, max_retries, retry_timeout, verbosity=0):
    """Introspect Baremetal Nodes

    :param clients: Application client object.
    :type clients: Object

    :param node_uuids: List of instance UUID(s).
    :type node_uuids: List

    :param run_validations: Enable or disable validations
    :type run_validations: Boolean

    :param concurrency: concurrency level
    :type concurrency: Integer

    :param verbosity: Verbosity level
    :type verbosity: Integer
    """

    with utils.TempDirs() as tmp:
        utils.run_ansible_playbook(
            playbook='cli-baremetal-introspect.yaml',
            inventory='localhost,',
            workdir=tmp,
            playbook_dir=constants.ANSIBLE_TRIPLEO_PLAYBOOKS,
            verbosity=verbosity,
            extra_vars={
                "node_uuids": node_uuids,
                "run_validations": run_validations,
                "concurrency": concurrency,
                "node_timeout": node_timeout,
                "max_retries": max_retries,
                "retry_timeout": retry_timeout,

            }
        )

    print('Successfully introspected nodes: {}'.format(node_uuids))


def introspect_manageable_nodes(clients, run_validations, concurrency,
                                node_timeout, max_retries, retry_timeout,
                                verbosity=0):
    """Introspect all manageable nodes

    :param clients: Application client object.
    :type clients: Object

    :param run_validations: Enable or disable validations
    :type run_validations: Boolean

    :param concurrency: Concurrency level
    :type concurrency: Integer

    :param node_timeout: Node timeout for introspection
    :type node_timeout: Integer

    :param max_retries: Max retries for introspection
    :type max_retries: Integer

    :param retry_timeout: Max timeout to wait between retries
    :type retry_timeout: Integer

    :param verbosity: Verbosity level
    :type verbosity: Integer
    """

    introspect(
        clients=clients,
        node_uuids=[
            i.uuid for i in clients.baremetal.node.list()
            if i.provision_state == "manageable" and not i.maintenance
        ],
        run_validations=run_validations,
        concurrency=concurrency,
        node_timeout=node_timeout,
        max_retries=max_retries,
        retry_timeout=retry_timeout,
        verbosity=verbosity
    )


def configure(clients, node_uuids, kernel_name='bm-deploy-kernel',
              ramdisk_name='bm-deploy-ramdisk', instance_boot_option=None,
              root_device=None, root_device_minimum_size=4,
              overwrite_root_device_hints=False):
    """Configure Node boot options.

    :param node_uuids: List of instance UUID(s).
    :type node_uuids: List

    :param kernel_name: Kernel to use
    :type kernel_name: String

    :param ramdisk_name: RAMDISK to use
    :type ramdisk_name: String

    :param instance_boot_option: Boot options to use
    :type instance_boot_option: String

    :param root_device: Path (name) of the root device.
    :type root_device: String

    :param root_device_minimum_size: Size of the given root device.
    :type root_device_minimum_size: Integer

    :param overwrite_root_device_hints: Whether to overwrite existing root
                                        device hints when `root_device` is
                                        used.
    :type overwrite_root_device_hints: Boolean
    """

    context = clients.tripleoclient.create_mistral_context()
    for node_uuid in node_uuids:
        boot_action = baremetal.ConfigureBootAction(
            node_uuid=node_uuid,
            kernel_name=kernel_name,
            ramdisk_name=ramdisk_name,
            instance_boot_option=instance_boot_option
        ).run(context=context)
        if boot_action:
            raise RuntimeError(boot_action)
        root_device_action = baremetal.ConfigureRootDeviceAction(
            node_uuid=node_uuid,
            root_device=root_device,
            minimum_size=root_device_minimum_size,
            overwrite=overwrite_root_device_hints
        )
        root_device_action.run(context=context)
    else:
        print('Successfully configured the nodes.')


def configure_manageable_nodes(clients, kernel_name='bm-deploy-kernel',
                               ramdisk_name='bm-deploy-ramdisk',
                               instance_boot_option=None,
                               root_device=None, root_device_minimum_size=4,
                               overwrite_root_device_hints=False):
    """Configure all manageable Nodes.

    kernel_name=parsed_args.deploy_kernel,
    ramdisk_name=parsed_args.deploy_ramdisk,
    instance_boot_option=parsed_args.instance_boot_option,
    root_device=parsed_args.root_device,
    root_device_minimum_size=parsed_args.root_device_minimum_size,
    overwrite_root_device_hints=(parsed_args.overwrite_root_device_hints)

    :param kernel_name: Kernel to use
    :type kernel_name: String

    :param ramdisk_name: RAMDISK to use
    :type ramdisk_name: String

    :param instance_boot_option: Boot options to use
    :type instance_boot_option: String

    :param root_device: Path (name) of the root device.
    :type root_device: String

    :param root_device_minimum_size: Size of the given root device.
    :type root_device_minimum_size: Integer

    :param overwrite_root_device_hints: Whether to overwrite existing root
                                        device hints when `root_device` is
                                        used.
    :type overwrite_root_device_hints: Boolean
    """

    configure(
        clients=clients,
        node_uuids=[
            i.uuid for i in clients.baremetal.node.list()
            if i.provision_state == "manageable" and not i.maintenance
        ],
        kernel_name=kernel_name,
        ramdisk_name=ramdisk_name,
        instance_boot_option=instance_boot_option,
        root_device=root_device,
        root_device_minimum_size=root_device_minimum_size,
        overwrite_root_device_hints=overwrite_root_device_hints
    )


def create_raid_configuration(clients, node_uuids, configuration,
                              verbosity=0):
    """Create RAID configuration on nodes.

    :param clients: application client object.
    :type clients: Object

    :param node_uuids: List of instance UUID(s).
    :type node_uuids: List

    :param configuration: RAID configuration object.
    :type configuration: Object

    :param verbosity: Verbosity level
    :type verbosity: Integer
    """

    with utils.TempDirs() as tmp:
        utils.run_ansible_playbook(
            playbook='cli-baremetal-raid.yaml',
            inventory='localhost,',
            workdir=tmp,
            playbook_dir=constants.ANSIBLE_TRIPLEO_PLAYBOOKS,
            verbosity=verbosity,
            extra_vars={
                'node_uuids': node_uuids,
                'raid_configuration': configuration
            }
        )

    print('Successfully configured RAID for nodes: {}'.format(node_uuids))


def discover_and_enroll(clients, ip_addresses, credentials, kernel_name,
                        ramdisk_name, instance_boot_option,
                        existing_nodes=None, ports=None):
    """Discover nodes and enroll baremetal nodes.

    :param clients: application client object.
    :type clients: Object

    :param ip_addresses: List of IP addresses.
    :type ip_addresses: List || String

    :param credentials: Credential information object
    :type credentials: Tuple

    :param kernel_name: Kernel to use
    :type kernel_name: String

    :param ramdisk_name: RAMDISK to use
    :type ramdisk_name: String

    :param instance_boot_option: Boot options to use
    :type instance_boot_option: String

    :param existing_nodes: List of nodes already discovered. If this is
                           undefined this object will be set to an empty
                           array.
    :type existing_nodes: List

    :param ports: List of ports, if no ports are provided the list of ports
                  will be limted to [623].
    :type ports: List

    :returns: List
    """

    if not ports:
        ports = [623]

    if not existing_nodes:
        existing_nodes = list()

    context = clients.tripleoclient.create_mistral_context()

    get_candiate_nodes = baremetal.GetCandidateNodes(
        ip_addresses,
        ports,
        credentials,
        existing_nodes
    )
    probed_nodes = list()
    for node in get_candiate_nodes.run(context=context):
        probed_nodes.append(
            baremetal.ProbeNode(**node).run(context=context)
        )
        print('Successfully probed node IP {}'.format(node['ip']))

    return register_or_update(
        clients=clients,
        nodes_json=probed_nodes,
        instance_boot_option=instance_boot_option,
        kernel_name=kernel_name,
        ramdisk_name=ramdisk_name
    )


def clean_nodes(node_uuids, verbosity=0):
    """Clean Baremetal Nodes

    :param node_uuids: List of instance UUID(s).
    :type node_uuids: List

    :param verbosity: Verbosity level
    :type verbosity: Integer
    """

    with utils.TempDirs() as tmp:
        utils.run_ansible_playbook(
            playbook='cli-baremetal-clean.yaml',
            inventory='localhost,',
            workdir=tmp,
            playbook_dir=constants.ANSIBLE_TRIPLEO_PLAYBOOKS,
            verbosity=verbosity,
            extra_vars={
                'node_uuids': node_uuids
            }
        )

    print('Successfully cleaned nodes: {}'.format(node_uuids))


def clean_manageable_nodes(clients, verbosity=0):
    """Clean all manageable Nodes

    :param clients: application client object.
    :type clients: Object

    :param verbosity: Verbosity level
    :type verbosity: Integer
    """

    clean_nodes(
        node_uuids=[
            i.uuid for i in clients.baremetal.node.list()
            if i.provision_state == "manageable" and not i.maintenance
        ],
        verbosity=verbosity
    )


def apply_bios_configuration(node_uuids, configuration, verbosity=0):
    """Apply BIOS settings on nodes.

    :param node_uuids: List of instance UUID(s).
    :type node_uuids: List

    :param configuration: BIOS configuration object.
    :type configuration: Object

    :param verbosity: Verbosity level
    :type verbosity: Integer
    """

    print('Applying BIOS settings for given nodes, this may take time')

    with utils.TempDirs() as tmp:
        utils.run_ansible_playbook(
            playbook='cli-baremetal-bios.yaml',
            inventory='localhost,',
            workdir=tmp,
            playbook_dir=constants.ANSIBLE_TRIPLEO_PLAYBOOKS,
            verbosity=verbosity,
            extra_vars={
                'node_uuids': node_uuids,
                'bios_configuration': configuration
            }
        )

    print('Successfully applied the BIOS for nodes: {}'.format(node_uuids))


def apply_bios_configuration_on_manageable_nodes(clients, configuration,
                                                 verbosity=0):
    """Apply BIOS settings on manageable nodes.

    :param clients: application client object.
    :type clients: Object

    :param configuration: BIOS configuration object.
    :type configuration: Object

    :param verbosity: Verbosity level
    :type verbosity: Integer
    """

    apply_bios_configuration(
        node_uuids=[
            i.uuid for i in clients.baremetal.node.list()
            if i.provision_state == "manageable" and not i.maintenance
        ],
        configuration=configuration,
        verbosity=verbosity
    )


def reset_bios_configuration(node_uuids, verbosity=0):
    """Reset BIOS settings on nodes.

    :param node_uuids: List of instance UUID(s).
    :type node_uuids: List

    :param verbosity: Verbosity level
    :type verbosity: Integer
    """

    with utils.TempDirs() as tmp:
        utils.run_ansible_playbook(
            playbook='cli-baremetal-bios-reset.yaml',
            inventory='localhost,',
            workdir=tmp,
            playbook_dir=constants.ANSIBLE_TRIPLEO_PLAYBOOKS,
            verbosity=verbosity,
            extra_vars={
                'node_uuids': node_uuids
            }
        )

    print('Successfully reset the BIOS for nodes: {}'.format(node_uuids))


def reset_bios_configuration_on_manageable_nodes(clients, verbosity=0):
    """Reset BIOS settings on manageable nodes.

    :param clients: application client object.
    :type clients: Object

    :param verbosity: Verbosity level
    :type verbosity: Integer
    """

    reset_bios_configuration(
        node_uuids=[
            i.uuid for i in clients.baremetal.node.list()
            if i.provision_state == "manageable" and not i.maintenance
        ],
        verbosity=verbosity
    )



#   Copyright 2018 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import mock

from tripleoclient.tests import fakes as ooofakes
from tripleoclient.tests.v1.overcloud_external_upgrade import fakes
from tripleoclient.v1 import overcloud_external_upgrade


class TestOvercloudExternalUpgradeRun(fakes.TestOvercloudExternalUpgradeRun):

    def setUp(self):
        super(TestOvercloudExternalUpgradeRun, self).setUp()

        # Get the command object to test
        app_args = mock.Mock()
        app_args.verbose_level = 1
        self.cmd = overcloud_external_upgrade.ExternalUpgradeRun(
            self.app, app_args)

        uuid4_patcher = mock.patch('uuid.uuid4', return_value="UUID4")
        self.mock_uuid4 = uuid4_patcher.start()
        self.addCleanup(self.mock_uuid4.stop)

    @mock.patch(
        'ansible_runner.runner_config.RunnerConfig',
        autospec=True,
        return_value=ooofakes.FakeRunnerConfig()
    )
    @mock.patch(
        'ansible_runner.Runner.run',
        return_value=ooofakes.fake_ansible_runner_run_return()
    )
    @mock.patch('tripleoclient.utils.run_ansible_playbook',
                autospec=True)
    @mock.patch('os.path.expanduser')
    @mock.patch('oslo_concurrency.processutils.execute')
    @mock.patch('six.moves.builtins.open')
    def test_upgrade_with_user_and_tags(self, mock_open, mock_execute,
                                        mock_expanduser, update_ansible,
                                        mock_run, mock_run_prepare):
        mock_expanduser.return_value = '/home/fake/'
        argslist = ['--ssh-user', 'tripleo-admin',
                    '--tags', 'ceph']
        verifylist = [
            ('ssh_user', 'tripleo-admin'),
            ('tags', 'ceph'),
        ]

        self.check_parser(self.cmd, argslist, verifylist)

    @mock.patch(
        'ansible_runner.runner_config.RunnerConfig',
        autospec=True,
        return_value=ooofakes.FakeRunnerConfig()
    )
    @mock.patch(
        'ansible_runner.Runner.run',
        return_value=ooofakes.fake_ansible_runner_run_return()
    )
    @mock.patch('tripleoclient.utils.run_ansible_playbook',
                autospec=True)
    @mock.patch('os.path.expanduser')
    @mock.patch('oslo_concurrency.processutils.execute')
    @mock.patch('six.moves.builtins.open')
    def test_upgrade_with_user_and_extra_vars(self, mock_open, mock_execute,
                                              mock_expanduser, update_ansible,
                                              mock_run, mock_run_prepare):
        mock_expanduser.return_value = '/home/fake/'
        argslist = ['--ssh-user', 'tripleo-admin',
                    '--extra-vars', 'key1=val1',
                    '--extra-vars', 'key2=val2']
        verifylist = [
            ('ssh_user', 'tripleo-admin'),
            ('extra_vars', ['key1=val1', 'key2=val2'])
        ]

        self.check_parser(self.cmd, argslist, verifylist)



#   Copyright 2018 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
from oslo_config import cfg
from oslo_log import log as logging

from osc_lib.i18n import _
from osc_lib import utils

from tripleoclient.exceptions import OvercloudUpgradeNotConfirmed

from tripleoclient import command
from tripleoclient import constants
from tripleoclient import utils as oooutils
from tripleoclient.v1.overcloud_deploy import DeployOvercloud
from tripleoclient.workflows import deployment
from tripleoclient.workflows import parameters

CONF = cfg.CONF


class UpgradePrepare(DeployOvercloud):
    """Run heat stack update for overcloud nodes to refresh heat stack outputs.

       The heat stack outputs are what we use later on to generate ansible
       playbooks which deliver the major upgrade workflow. This is used as the
       first step for a major upgrade of your overcloud.
    """

    operation = "Prepare"

    template = constants.UPGRADE_PREPARE_ENV

    forbidden_params = []

    log = logging.getLogger(__name__ + ".UpgradePrepare")

    def get_parser(self, prog_name):
        parser = super(UpgradePrepare, self).get_parser(prog_name)
        parser.add_argument('-y', '--yes', default=False,
                            action='store_true',
                            help=_("Use -y or --yes to skip the confirmation "
                                   "required before any upgrade "
                                   "operation. Use this with caution! "),
                            )
        return parser

    def take_action(self, parsed_args):
        logging.register_options(CONF)
        logging.setup(CONF, '')
        self.log.debug("take_action(%s)" % parsed_args)
        oooutils.ensure_run_as_normal_user()

        if (not parsed_args.yes
                and not oooutils.prompt_user_for_confirmation(
                    constants.UPGRADE_PROMPT, self.log)):
            raise OvercloudUpgradeNotConfirmed(constants.UPGRADE_NO)

        # Throw deprecation warning if service is enabled and
        # ask user if upgrade should still be continued.
        if parsed_args.environment_files:
            oooutils.check_deprecated_service_is_enabled(
                parsed_args.environment_files)

        clients = self.app.client_manager

        stack = oooutils.get_stack(clients.orchestration,
                                   parsed_args.stack)

        stack_name = stack.stack_name

        # In case of update and upgrade we need to force the
        # config_download to false. The heat stack update will be performed
        # by DeployOvercloud class but skipping the config download part.
        parsed_args.config_download = False
        # Add the template attribute environment to set noops etc
        templates_dir = (parsed_args.templates or
                         constants.TRIPLEO_HEAT_TEMPLATES)
        parsed_args.environment_files = oooutils.prepend_environment(
            parsed_args.environment_files, templates_dir,
            self.template)
        # Parse all environment files looking for undesired
        # parameters
        parameters.check_forbidden_params(self.log,
                                          parsed_args.environment_files,
                                          self.forbidden_params)
        super(UpgradePrepare, self).take_action(parsed_args)

        # enable ssh admin for Ansible-via-Mistral as that's done only
        # when config_download is true
        deployment.get_hosts_and_enable_ssh_admin(
            stack,
            parsed_args.overcloud_ssh_network,
            parsed_args.overcloud_ssh_user,
            self.get_key_pair(parsed_args),
            parsed_args.overcloud_ssh_port_timeout,
            verbosity=oooutils.playbook_verbosity(self=self)
        )

        self.log.info("Completed Overcloud Upgrade {} for stack "
                      "{}".format(self.operation, stack_name))


class UpgradeRun(command.Command):
    """Run major upgrade ansible playbooks on Overcloud nodes

       This will run the major upgrade ansible playbooks on the overcloud.
       By default all playbooks are executed, that is the
       upgrade_steps_playbook.yaml then the deploy_steps_playbook.yaml and
       then the post_upgrade_steps_playbook.yaml.
       The upgrade playbooks are made available after completion of the
       'overcloud upgrade prepare' command. This 'overcloud upgrade run'
       command is the second step in the major upgrade workflow.
    """

    log = logging.getLogger(__name__ + ".UpgradeRun")

    def get_parser(self, prog_name):
        parser = super(UpgradeRun, self).get_parser(prog_name)
        parser.add_argument(
            '--limit',
            action='store',
            required=True,
            help=_("A string that identifies a single node or comma-separated"
                   "list of nodes the config-download Ansible playbook "
                   "execution will be limited to. For example: --limit"
                   " \"compute-0,compute-1,compute-5\".")
        )
        parser.add_argument('--playbook',
                            nargs="*",
                            default=None,
                            help=_("Ansible playbook to use for the minor"
                                   " update. Can be used multiple times."
                                   " Set this to each of those playbooks in"
                                   " consecutive invocations of this command"
                                   " if you prefer to run them manually."
                                   " Note: make sure to run all playbooks so"
                                   " that all services are updated and running"
                                   " with the target version configuration.")
                            )
        parser.add_argument('--static-inventory',
                            dest='static_inventory',
                            action="store",
                            default=None,
                            help=_('Path to an existing ansible inventory to '
                                   'use. If not specified, one will be '
                                   'generated in '
                                   '~/tripleo-ansible-inventory.yaml')
                            )
        parser.add_argument("--ssh-user",
                            dest="ssh_user",
                            action="store",
                            default="tripleo-admin",
                            help=_("DEPRECATED: Only tripleo-admin should be "
                                   "used as ssh user.")
                            )
        parser.add_argument('--tags',
                            dest='tags',
                            action="store",
                            default="",
                            help=_('A string specifying the tag or comma '
                                   'separated list of tags to be passed '
                                   'as --tags to ansible-playbook.')
                            )
        parser.add_argument('--skip-tags',
                            dest='skip_tags',
                            action="store",
                            default="",
                            help=_('A string specifying the tag or comma '
                                   'separated list of tags to be passed '
                                   'as --skip-tags to ansible-playbook. '
                                   'The currently supported values are '
                                   '\'validation\' and \'pre-upgrade\'. '
                                   'In particular \'validation\' is useful '
                                   'if you must re-run following a failed '
                                   'upgrade and some services cannot be '
                                   'started. ')
                            )
        parser.add_argument('--stack', dest='stack',
                            help=_('Name or ID of heat stack '
                                   '(default=Env: OVERCLOUD_STACK_NAME)'),
                            default=utils.env('OVERCLOUD_STACK_NAME',
                                              default='overcloud')
                            )
        parser.add_argument('--no-workflow', dest='no_workflow',
                            action='store_true',
                            default=True,
                            help=_('This option no longer has any effect.')
                            )
        parser.add_argument('-y', '--yes', default=False,
                            action='store_true',
                            help=_("Use -y or --yes to skip the confirmation "
                                   "required before any upgrade "
                                   "operation. Use this with caution! ")
                            )
        parser.add_argument(
            '--ansible-forks',
            action='store',
            default=None,
            type=int,
            help=_('The number of Ansible forks to use for the'
                   ' config-download ansible-playbook command.')
        )
        return parser

    def take_action(self, parsed_args):
        logging.register_options(CONF)
        logging.setup(CONF, '')
        self.log.debug("take_action(%s)" % parsed_args)
        oooutils.ensure_run_as_normal_user()

        if (not parsed_args.yes
                and not oooutils.prompt_user_for_confirmation(
                    constants.UPGRADE_PROMPT, self.log)):
            raise OvercloudUpgradeNotConfirmed(constants.UPGRADE_NO)

        # NOTE(cloudnull): The string option "all" was a special default
        #                  that is no longer relevant. To retain compatibility
        #                  this condition has been put in place.
        if not parsed_args.playbook or parsed_args.playbook == ['all']:
            playbook = constants.MAJOR_UPGRADE_PLAYBOOKS
        else:
            playbook = parsed_args.playbook

        _, ansible_dir = self.get_ansible_key_and_dir(
            no_workflow=parsed_args.no_workflow,
            stack=parsed_args.stack,
            orchestration=self.app.client_manager.orchestration
        )
        deployment.config_download(
            log=self.log,
            clients=self.app.client_manager,
            stack=oooutils.get_stack(
                self.app.client_manager.orchestration,
                parsed_args.stack
            ),
            output_dir=ansible_dir,
            verbosity=oooutils.playbook_verbosity(self=self),
            ansible_playbook_name=playbook,
            inventory_path=oooutils.get_tripleo_ansible_inventory(
                parsed_args.static_inventory,
                parsed_args.ssh_user,
                parsed_args.stack,
                return_inventory_file_path=True
            ),
            tags=parsed_args.tags,
            skip_tags=parsed_args.skip_tags,
            limit_hosts=oooutils.playbook_limit_parse(
                limit_nodes=parsed_args.limit
            ),
            forks=parsed_args.ansible_forks
        )
        self.log.info("Completed Overcloud Major Upgrade Run.")


class UpgradeConverge(UpgradePrepare):
    """Major upgrade converge - reset Heat resources in the stored plan

       This is the last step for completion of a overcloud major
       upgrade.  The main task is updating the plan and stack to
       unblock future stack updates. For the major upgrade workflow we
       have set specific values for some stack Heat resources. This
       unsets those back to their default values.
    """
    operation = "Converge"

    template = constants.UPGRADE_CONVERGE_ENV

    forbidden_params = constants.UPGRADE_CONVERGE_FORBIDDEN_PARAMS

    log = logging.getLogger(__name__ + ".UpgradeConverge")



#   Copyright 2017 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

from argparse import _StoreAction
import logging

from osc_lib.command import command
from osc_lib import exceptions as oscexc

from tripleo_common.utils import config

from tripleoclient import exceptions
from tripleoclient import utils


class Command(command.Command):

    log = logging.getLogger(__name__ + ".Command")

    def run(self, parsed_args):
        utils.store_cli_param(self.cmd_name, parsed_args)
        try:
            super(Command, self).run(parsed_args)
        except (oscexc.CommandError, exceptions.Base):
            raise
        except Exception:
            self.log.exception("Exception occured while running the command")
            raise

    @staticmethod
    def get_ansible_key_and_dir(no_workflow, stack, orchestration):
        """Return the ansible directory and key path.

        :param no_workflow: Enable or disable the mistral workflow code path.
        :type no_workflow: Boolean

        :oaram stack: Name of a given stack to run against.
        :type stack: String

        :param orchestration: Orchestration client object.
        :type orchestration: Object

        :returns: Tuple
        """

        if no_workflow:
            key = utils.get_key(stack=stack)
            stack_config = config.Config(orchestration)
            with utils.TempDirs(chdir=False) as tmp:
                stack_config.write_config(
                    stack_config.fetch_config(stack),
                    stack,
                    tmp
                )
                return key, tmp
        else:
            # Assumes execution will take place from within a mistral
            # container.
            key = '.ssh/tripleo-admin-rsa'
            return key, None

    def get_key_pair(self, parsed_args):
        """Autodetect or return a user defined key file.

        :param parsed_args: An argparse object.
        :type parsed_args: Object

        :returns: String
        """

        if not parsed_args.overcloud_ssh_key:
            key = utils.get_key(
                stack=parsed_args.stack,
                needs_pair=True
            )
            if not key:
                raise oscexc.CommandError(
                    'No key pair found, set the ssh key using'
                    'the --overcloud-ssh-key switch.'
                )
            return key
        else:
            return parsed_args.overcloud_ssh_key


class Lister(Command, command.Lister):
    pass


class ShowOne(Command, command.ShowOne):
    pass


class DeprecatedActionStore(_StoreAction):
    """To deprecated an option an store the value"""
    log = logging.getLogger(__name__)

    def __call__(self, parser, namespace, values, option_string=None):
        """Display the warning message"""
        if len(self.option_strings) == 1:
            message = 'The option {option} is deprecated, it will be removed'\
                      ' in a future version'.format(
                          option=self.option_strings[0])
        else:
            option = ', '.join(self.option_strings)
            message = 'The options {option} is deprecated, it will be removed'\
                      ' in a future version'.format(option=option)

        self.log.warning(message)
        super(DeprecatedActionStore, self).__call__(
            parser, namespace, values, option_string)



#   Copyright 2015 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import mock

from tripleoclient import exceptions
from tripleoclient.tests import test_utils
from tripleoclient.tests.v1 import test_plugin
from tripleoclient import utils
from tripleoclient.v1 import overcloud_profiles


@mock.patch.object(utils, 'assign_and_verify_profiles', autospec=True)
class TestMatchProfiles(test_plugin.TestPluginV1):
    def setUp(self):
        super(TestMatchProfiles, self).setUp()
        self.cmd = overcloud_profiles.MatchProfiles(self.app, None)
        self.app.client_manager.tripleoclient = mock.Mock()
        self.app.client_manager.baremetal = mock.Mock()
        self.app.client_manager.compute = mock.Mock()
        self.flavors = [
            test_utils.FakeFlavor('compute'),
            test_utils.FakeFlavor('control'),
        ]
        self.app.client_manager.compute.flavors.list.return_value = (
            self.flavors)

    def test_ok(self, mock_assign):
        mock_assign.return_value = (0, 0)

        arglist = [
            '--compute-flavor', 'compute',
            '--compute-scale', '3',
            '--control-flavor', 'control',
            '--control-scale', '1',
        ]
        parsed_args = self.check_parser(self.cmd, arglist, [])

        self.cmd.take_action(parsed_args)

        mock_assign.assert_called_once_with(
            self.app.client_manager.baremetal,
            {'compute': (self.flavors[0], 3),
             'control': (self.flavors[1], 1)},
            assign_profiles=True, dry_run=False)

    def test_failed(self, mock_assign):
        mock_assign.return_value = (2, 0)

        arglist = [
            '--compute-flavor', 'compute',
            '--compute-scale', '3',
            '--control-flavor', 'control',
            '--control-scale', '1',
        ]
        parsed_args = self.check_parser(self.cmd, arglist, [])

        self.assertRaises(exceptions.ProfileMatchingError,
                          self.cmd.take_action, parsed_args)

        mock_assign.assert_called_once_with(
            self.app.client_manager.baremetal,
            {'compute': (self.flavors[0], 3),
             'control': (self.flavors[1], 1)},
            assign_profiles=True, dry_run=False)

    def test_dry_run(self, mock_assign):
        mock_assign.return_value = (0, 0)

        arglist = [
            '--compute-flavor', 'compute',
            '--compute-scale', '3',
            '--control-flavor', 'control',
            '--control-scale', '1',
            '--dry-run'
        ]
        parsed_args = self.check_parser(self.cmd, arglist, [])

        self.cmd.take_action(parsed_args)

        mock_assign.assert_called_once_with(
            self.app.client_manager.baremetal,
            {'compute': (self.flavors[0], 3),
             'control': (self.flavors[1], 1)},
            assign_profiles=True, dry_run=True)


class TestListProfiles(test_plugin.TestPluginV1):
    def setUp(self):
        super(TestListProfiles, self).setUp()
        self.cmd = overcloud_profiles.ListProfiles(self.app, None)
        self.app.client_manager.tripleoclient = mock.Mock()
        self.app.client_manager.baremetal = mock.Mock()
        self.app.client_manager.compute = mock.Mock()
        self.nodes = [
            mock.Mock(uuid='uuid1', provision_state='active',
                      properties={}, maintenance=False),
            mock.Mock(uuid='uuid2', provision_state='enroll',
                      properties={'capabilities': 'profile:compute'},
                      maintenance=False),
            mock.Mock(uuid='uuid3', provision_state='available',
                      properties={'capabilities': 'profile:compute,'
                                  'compute_profile:1,control_profile:true'},
                      maintenance=False),
            mock.Mock(uuid='uuid4', provision_state='available',
                      properties={'capabilities': 'profile:compute,'
                                  'compute_profile:0'}, maintenance=False),
            mock.Mock(uuid='uuid5', provision_state='available',
                      properties={}, maintenance=False),
            mock.Mock(uuid='uuid6', provision_state='available',
                      properties={}, maintenance=False),
            mock.Mock(uuid='uuid7', provision_state='active',
                      properties={}, maintenance=True),
        ]
        self.hypervisors = [
            mock.Mock(hypervisor_type='ironic',
                      hypervisor_hostname='uuid%d' % i,
                      status='enabled', state='up')
            for i in range(1, 6)
        ]
        self.hypervisors[-1].status = 'disabled'
        self.bm_client = self.app.client_manager.baremetal
        self.bm_client.node.list.return_value = self.nodes
        self.compute_client = self.app.client_manager.compute
        self.compute_client.hypervisors.list.return_value = self.hypervisors

    def test_list(self):
        parsed_args = self.check_parser(self.cmd, [], [])
        result = self.cmd.take_action(parsed_args)
        self.assertEqual(5, len(result[0]))
        self.assertEqual(
            [('uuid1', self.nodes[0].name, 'active', None, ''),
             ('uuid3', self.nodes[2].name, 'available', 'compute',
              'compute, control'),
             ('uuid4', self.nodes[3].name, 'available', 'compute', '')],
            result[1])

    def test_all(self):
        parsed_args = self.check_parser(self.cmd, ['--all'], [('all', True)])
        result = self.cmd.take_action(parsed_args)
        self.assertEqual(6, len(result[0]))
        self.assertEqual(
            [('uuid1', self.nodes[0].name, 'active', None, '', ''),
             ('uuid2', self.nodes[1].name, 'enroll', 'compute', '',
              'Provision state enroll'),
             ('uuid3', self.nodes[2].name, 'available', 'compute',
              'compute, control', ''),
             ('uuid4', self.nodes[3].name, 'available', 'compute', '', ''),
             ('uuid5', self.nodes[4].name, 'available', None, '',
              'Compute service disabled'),
             ('uuid6', self.nodes[5].name, 'available', None, '',
              'No hypervisor record'),
             ('uuid7', self.nodes[6].name, 'active', None, '',
              'Maintenance')],
            result[1])



#   Copyright 2018 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

from tripleoclient.config.undercloud import UndercloudConfig
from tripleoclient.tests import base


class TestUndercloudConfig(base.TestCase):
    def setUp(self):
        super(TestUndercloudConfig, self).setUp()
        # Get the class object to test
        self.config = UndercloudConfig()

    def test_get_base_opts(self):
        ret = self.config.get_base_opts()
        expected = ['additional_architectures',
                    'certificate_generation_ca',
                    'clean_nodes',
                    'cleanup',
                    'container_cli',
                    'container_healthcheck_disabled',
                    'container_images_file',
                    'container_insecure_registries',
                    'container_registry_mirror',
                    'custom_env_files',
                    'deployment_user',
                    'discovery_default_driver',
                    'enable_node_discovery',
                    'enable_routed_networks',
                    'enable_swift_encryption',
                    'enabled_hardware_types',
                    'generate_service_certificate',
                    'heat_container_image',
                    'heat_native',
                    'hieradata_override',
                    'inspection_extras',
                    'inspection_interface',
                    'inspection_runbench',
                    'ipa_otp',
                    'ipv6_address_mode',
                    'ipxe_enabled',
                    'ironic_default_network_interface',
                    'ironic_enabled_network_interfaces',
                    'local_interface',
                    'local_ip',
                    'local_mtu',
                    'local_subnet',
                    'net_config_override',
                    'networks_file',
                    'output_dir',
                    'overcloud_domain_name',
                    'roles_file',
                    'scheduler_max_attempts',
                    'service_principal',
                    'subnets',
                    'templates',
                    'undercloud_admin_host',
                    'undercloud_debug',
                    'undercloud_enable_selinux',
                    'undercloud_hostname',
                    'undercloud_log_file',
                    'undercloud_nameservers',
                    'undercloud_ntp_servers',
                    'undercloud_public_host',
                    'undercloud_service_certificate',
                    'undercloud_timezone']
        self.assertEqual(expected, [x.name for x in ret])

    def test_get_opts(self):
        ret = self.config.get_opts()
        expected = ['additional_architectures',
                    'certificate_generation_ca',
                    'clean_nodes',
                    'cleanup',
                    'container_cli',
                    'container_healthcheck_disabled',
                    'container_images_file',
                    'container_insecure_registries',
                    'container_registry_mirror',
                    'custom_env_files',
                    'deployment_user',
                    'discovery_default_driver',
                    'enable_cinder',
                    'enable_ironic',
                    'enable_ironic_inspector',
                    'enable_mistral',
                    'enable_node_discovery',
                    'enable_nova',
                    'enable_novajoin',
                    'enable_routed_networks',
                    'enable_swift',
                    'enable_swift_encryption',
                    'enable_telemetry',
                    'enable_tempest',
                    'enable_validations',
                    'enable_zaqar',
                    'enabled_hardware_types',
                    'generate_service_certificate',
                    'heat_container_image',
                    'heat_native',
                    'hieradata_override',
                    'inspection_extras',
                    'inspection_interface',
                    'inspection_runbench',
                    'ipa_otp',
                    'ipv6_address_mode',
                    'ipxe_enabled',
                    'ironic_default_network_interface',
                    'ironic_enabled_network_interfaces',
                    'local_interface',
                    'local_ip',
                    'local_mtu',
                    'local_subnet',
                    'net_config_override',
                    'networks_file',
                    'output_dir',
                    'overcloud_domain_name',
                    'roles_file',
                    'scheduler_max_attempts',
                    'service_principal',
                    'subnets',
                    'templates',
                    'undercloud_admin_host',
                    'undercloud_debug',
                    'undercloud_enable_selinux',
                    'undercloud_hostname',
                    'undercloud_log_file',
                    'undercloud_nameservers',
                    'undercloud_ntp_servers',
                    'undercloud_public_host',
                    'undercloud_service_certificate',
                    'undercloud_timezone']
        self.assertEqual(expected, [x.name for x in ret])

    def test_get_subnet_opts(self):
        expected = ['cidr',
                    'dhcp_end',
                    'dhcp_exclude',
                    'dhcp_start',
                    'dns_nameservers',
                    'gateway',
                    'host_routes',
                    'inspection_iprange',
                    'masquerade']

        ret = self.config.get_local_subnet_opts()
        self.assertEqual(expected, [x.name for x in ret])

        ret = self.config.get_remote_subnet_opts()
        self.assertEqual(expected, [x.name for x in ret])

    def test_get_undercloud_service_opts(self):
        ret = self.config.get_undercloud_service_opts()
        expected = {'enable_cinder': False,
                    'enable_ironic': True,
                    'enable_ironic_inspector': True,
                    'enable_mistral': False,
                    'enable_nova': False,
                    'enable_novajoin': False,
                    'enable_telemetry': False,
                    'enable_swift': True,
                    'enable_tempest': True,
                    'enable_validations': True,
                    'enable_zaqar': False}
        self.assertEqual(sorted(expected.keys()), [x.name for x in ret])
        for x in ret:
            self.assertEqual(expected[x.name], x.default, "%s config not %s" %
                             (x.name, expected[x.name]))



#   Copyright 2013 Nebula Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#


"""Exception definitions"""


class Base(Exception):
    """Base TripleO exception."""


class WorkflowServiceError(Base):
    """The service type is unknown"""


class WebSocketConnectionClosed(Base):
    """Websocket connection is closed before wait for messages"""


class NotFound(Base):
    """Resource not found"""


class LookupError(Base):
    """Lookup Error"""


class DeploymentError(Base):
    """Deployment failed"""


class PlanEnvWorkflowError(Base):
    """Plan Environment workflow has failed"""


class ConfigDownloadInProgress(Base):
    """Unable to deploy as config download already in progress"""

    msg_format = ("Config download already in progress with "
                  "execution id {} for stack {}")

    def __init__(self, execution_id='', stack=''):
        message = self.msg_format.format(execution_id, stack)
        super(ConfigDownloadInProgress, self).__init__(message)


class RootUserExecution(Base):
    """Command was executed by a root user"""


class InvalidConfiguration(Base, ValueError):
    """Invalid parameters were specified for the deployment"""


class IntrospectionError(Base):
    """Introspection failed"""


class RegisterOrUpdateError(WorkflowServiceError):
    """Introspection failed"""


class NodeProvideError(WorkflowServiceError):
    """Node Provide failed."""


class NodeConfigurationError(WorkflowServiceError):
    """Node Configuration failed."""


class ProfileMatchingError(Base):
    """Failed to validate or assign node profiles"""


class PlanCreationError(Base):
    """Plan creation failed"""


class PlanExportError(Base):
    """Plan export failed"""


class WorkflowActionError(Base):
    """Workflow action failed"""
    msg_format = "Action {} execution failed: {}"

    def __init__(self, action='', output=''):
        message = self.msg_format.format(action, output)
        super(WorkflowActionError, self).__init__(message)


class DownloadError(Base):
    """Download attempt failed"""


class LogFetchError(Base):
    """Fetching logs failed"""


class ContainerDeleteFailed(Base):
    """Container deletion failed"""


class UndercloudUpgradeNotConfirmed(Base):
    """Undercloud upgrade security question not confirmed."""


class OvercloudUpdateNotConfirmed(Base):
    """Overcloud Update security question not confirmed."""


class OvercloudUpgradeNotConfirmed(Base):
    """Overcloud Update security question not confirmed."""


class CellExportError(Base):
    """Cell export failed"""


class BannedParameters(Base):
    """Some of the environment parameters provided should be removed"""



#   Copyright 2018 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import argparse
import logging

from cliff import command
from osc_lib.i18n import _

from tripleoclient import constants
from tripleoclient import utils

# For ansible.cfg generation
from tripleo_common.actions import ansible


class GenerateAnsibleConfig(command.Command):
    """Generate the default ansible.cfg for deployments."""

    log = logging.getLogger(__name__ + ".GenerateAnsibleConfig")

    def get_parser(self, prog_name):
        parser = argparse.ArgumentParser(
            description=self.get_description(),
            prog=prog_name,
            add_help=False
        )
        # TODO(bogdando): drop that once using oslo.privsep
        parser.add_argument(
            '--deployment-user',
            dest='deployment_user',
            default='stack',
            help=_('User who executes the tripleo config generate command. '
                   'Defaults to stack.')
        )
        # TODO(bogdando): find a better UNDERCLOUD_OUTPUT_DIR constant name
        # Add more params as far as the imported ansible actions support it
        parser.add_argument('--output-dir',
                            dest='output_dir',
                            help=_("Directory to output ansible.cfg and "
                                   "ansible.log files."),
                            default=constants.UNDERCLOUD_OUTPUT_DIR)

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        if utils.get_deployment_user() != parsed_args.deployment_user:
            self.log.warning(
                _('The --deployment-user value %s does not '
                  'match the user name executing this command!') %
                parsed_args.deployment_user)

        ansible.write_default_ansible_cfg(parsed_args.output_dir,
                                          parsed_args.deployment_user,
                                          ssh_private_key=None)



#   Copyright 2018 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

from oslo_config import cfg
from tripleoclient import constants


class BaseConfig(object):

    def sort_opts(self, opts):
        """Sort oslo config options by name

        :param opts: list of olo cfg opts
        :return list - sorted by name
        """
        def sort_cfg(cfg):
            return cfg.name
        return sorted(opts, key=sort_cfg)

    def get_base_opts(self):
        _opts = [
            # TODO(aschultz): rename undercloud_output_dir
            cfg.StrOpt('output_dir',
                       default=constants.UNDERCLOUD_OUTPUT_DIR,
                       help=(
                           'Directory to output state, processed heat '
                           'templates, ansible deployment files.'),
                       ),
            cfg.BoolOpt('cleanup',
                        default=True,
                        help=('Cleanup temporary files. Setting this to '
                              'False will leave the temporary files used '
                              'during deployment in place after the command '
                              'is run. This is useful for debugging the '
                              'generated files or if errors occur.'),
                        ),
        ]
        return self.sort_opts(_opts)



#   Copyright 2018 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import copy

from osc_lib.i18n import _
from oslo_config import cfg

from tripleoclient import constants

from tripleoclient.config.standalone import StandaloneConfig

CONF = cfg.CONF

# Control plane network name
SUBNETS_DEFAULT = ['ctlplane-subnet']

CIDR_HELP_STR = _(
    'Network CIDR for the Neutron-managed subnet for Overcloud instances.')
DHCP_START_HELP_STR = _(
    'Start of DHCP allocation range for PXE and DHCP of Overcloud instances '
    'on this network.')
DHCP_END_HELP_STR = _(
    'End of DHCP allocation range for PXE and DHCP of Overcloud instances on '
    'this network.')
DHCP_EXCLUDE_HELP_STR = _(
    'List of IP addresses or IP ranges to exclude from the subnets allocation '
    'pool. Example: 192.168.24.50,192.168.24.80-192.168.24.90')
INSPECTION_IPRANGE_HELP_STR = _(
    'Temporary IP range that will be given to nodes on this network during '
    'the inspection process. Should not overlap with the range defined by '
    'dhcp_start and dhcp_end, but should be in the same ip subnet.')
GATEWAY_HELP_STR = _(
    'Network gateway for the Neutron-managed network for Overcloud instances '
    'on this network.')
MASQUERADE_HELP_STR = _(
    'The network will be masqueraded for external access.')
HOST_ROUTES_HELP_STR = _(
    'Host routes for the Neutron-managed subnet for the Overcloud instances '
    'on this network. The host routes on the local_subnet will also be '
    'configured on the undercloud.')
DNS_NAMESERVERS_HELP_STR = _(
    'DNS nameservers for the Neutron-managed subnet for the Overcloud '
    'instances on this network. If no nameservers are defined for the subnet, '
    'the nameservers defined for undercloud_nameservers will be used.')

# Deprecated options
_deprecated_opt_network_gateway = [cfg.DeprecatedOpt(
    'network_gateway', group='DEFAULT')]
_deprecated_opt_network_cidr = [cfg.DeprecatedOpt(
    'network_cidr', group='DEFAULT')]
_deprecated_opt_dhcp_start = [cfg.DeprecatedOpt(
    'dhcp_start', group='DEFAULT')]
_deprecated_opt_dhcp_end = [cfg.DeprecatedOpt('dhcp_end', group='DEFAULT')]
_deprecated_opt_inspection_iprange = [cfg.DeprecatedOpt(
    'inspection_iprange', group='DEFAULT')]


class UndercloudConfig(StandaloneConfig):
    def get_undercloud_service_opts(self):
        return super(UndercloudConfig, self).get_enable_service_opts(
            cinder=False,
            ironic=True,
            ironic_inspector=True,
            mistral=False,
            nova=False,
            novajoin=False,
            swift=True,
            tempest=True,
            telemetry=False,
            validations=True,
            zaqar=False)

    def get_base_opts(self):
        _base_opts = super(UndercloudConfig, self).get_base_opts()
        _opts = [
            cfg.StrOpt('undercloud_log_file',
                       default=constants.UNDERCLOUD_LOG_FILE,
                       help=_(
                           'The path to a log file to store the '
                           'undercloud install/upgrade logs.'),
                       ),
            cfg.StrOpt('undercloud_hostname',
                       help=_(
                           'Fully qualified hostname (including domain) to '
                           'set on the Undercloud. If left unset, the current '
                           'hostname will be used, but the user is '
                           'responsible for configuring all system hostname '
                           'settings appropriately.  If set, the undercloud '
                           'install will configure all system hostname '
                           'settings.'),
                       ),
            cfg.StrOpt('local_ip',
                       default='192.168.24.1/24',
                       help=_(
                           'IP information for the interface on the '
                           'Undercloud that will be handling the PXE boots '
                           'and DHCP for Overcloud instances.  The IP portion '
                           'of the value will be assigned to the network '
                           'interface defined by local_interface, with the '
                           'netmask defined by the prefix portion of the '
                           'value.')
                       ),
            cfg.StrOpt('undercloud_public_host',
                       deprecated_name='undercloud_public_vip',
                       default='192.168.24.2',
                       help=_(
                           'Virtual IP or DNS address to use for the public '
                           'endpoints of Undercloud services. Only used '
                           'with SSL.')
                       ),
            cfg.StrOpt('undercloud_admin_host',
                       deprecated_name='undercloud_admin_vip',
                       default='192.168.24.3',
                       help=_(
                           'Virtual IP or DNS address to use for the admin '
                           'endpoints of Undercloud services. Only used '
                           'with SSL.')
                       ),
            cfg.ListOpt('undercloud_nameservers',
                        default=[],
                        help=_(
                            'DNS nameserver(s). Use for the undercloud '
                            'node and for the overcloud nodes. (NOTE: To use '
                            'different nameserver(s) for the overcloud, '
                            'override the DnsServers parameter in overcloud '
                            'environment.)'),
                        ),
            cfg.ListOpt('undercloud_ntp_servers',
                        default=['0.pool.ntp.org', '1.pool.ntp.org',
                                 '2.pool.ntp.org', '3.pool.ntp.org'],
                        help=_('List of ntp servers to use.')),
            cfg.StrOpt('undercloud_timezone', default=None,
                       help=_('Host timezone to be used. If no timezone is '
                              'specified, the existing timezone configuration '
                              'is used.')),
            cfg.StrOpt('overcloud_domain_name',
                       default='localdomain',
                       help=_(
                           'DNS domain name to use when deploying the '
                           'overcloud. The overcloud parameter "CloudDomain" '
                           'must be set to a matching value.')
                       ),
            cfg.ListOpt('subnets',
                        default=SUBNETS_DEFAULT,
                        help=_(
                            'List of routed network subnets for '
                            'provisioning and introspection. Comma '
                            'separated list of names/tags. For each network '
                            'a section/group needs to be added to the '
                            'configuration file with these parameters set: '
                            'cidr, dhcp_start, dhcp_end, inspection_iprange, '
                            'gateway and masquerade_network. Note: The '
                            'section/group must be placed before or after '
                            'any other section. (See the example section '
                            '[ctlplane-subnet] in the sample configuration '
                            'file.)')),
            cfg.StrOpt('local_subnet',
                       default=SUBNETS_DEFAULT[0],
                       help=_(
                           'Name of the local subnet, where the PXE boot and '
                           'DHCP interfaces for overcloud instances is '
                           'located. The IP address of the '
                           'local_ip/local_interface should reside '
                           'in this subnet.')),
            cfg.StrOpt('undercloud_service_certificate',
                       default='',
                       help=_(
                           'Certificate file to use for OpenStack service SSL '
                           'connections.  Setting this enables SSL for the '
                           'OpenStack API endpoints, leaving it unset '
                           'disables SSL.')
                       ),
            cfg.BoolOpt('generate_service_certificate',
                        default=True,
                        help=_(
                            'When set to True, an SSL certificate will be '
                            'generated as part of the undercloud install and '
                            'this certificate will be used in place of the '
                            'value for undercloud_service_certificate.  The '
                            'resulting certificate will be written to '
                            '/etc/pki/tls/private/overcloud_endpoint.pem. This'
                            ' certificate is signed by CA selected by the '
                            '"certificate_generation_ca" option.')
                        ),
            cfg.StrOpt('certificate_generation_ca',
                       default='local',
                       help=_(
                           'The certmonger nickname of the CA from which '
                           'the certificate will be requested. This is used '
                           'only if the generate_service_certificate option '
                           'is set. Note that if the "local" CA is selected '
                           'the certmonger\'s local CA certificate will be '
                           'extracted to /etc/pki/ca-trust/source/anchors/'
                           'cm-local-ca.pem and subsequently added to the '
                           'trust chain.')
                       ),
            cfg.StrOpt('service_principal',
                       default='',
                       help=_(
                           'The kerberos principal for the service that will '
                           'use the certificate. This is only needed if your '
                           'CA requires a kerberos principal. e.g. with '
                           'FreeIPA.')
                       ),
            cfg.StrOpt('local_interface',
                       default='eth1',
                       help=_('Network interface on the Undercloud that will '
                              'be handling the PXE boots and DHCP for '
                              'Overcloud instances.')
                       ),
            cfg.IntOpt('local_mtu',
                       default=1500,
                       help=_('MTU to use for the local_interface.')
                       ),
            cfg.StrOpt('inspection_interface',
                       default='br-ctlplane',
                       deprecated_name='discovery_interface',
                       help=_(
                           'Network interface on which inspection dnsmasq '
                           'will listen.  If in doubt, use the default value.')
                       ),
            cfg.BoolOpt('inspection_extras',
                        default=True,
                        help=_(
                            'Whether to enable extra hardware collection '
                            'during the inspection process. Requires '
                            'python-hardware or python-hardware-detect '
                            'package on the introspection image.')),
            cfg.BoolOpt('inspection_runbench',
                        default=False,
                        deprecated_name='discovery_runbench',
                        help=_(
                            'Whether to run benchmarks when inspecting '
                            'nodes. Requires inspection_extras set to True.')
                        ),
            cfg.BoolOpt('enable_node_discovery',
                        default=False,
                        help=_(
                            'Makes ironic-inspector enroll any unknown node '
                            'that PXE-boots introspection ramdisk in Ironic. '
                            'By default, the "fake" driver is used for new '
                            'nodes (it is automatically enabled when this '
                            'option is set to True). Set '
                            'discovery_default_driver to override. '
                            'Introspection rules can also be used to specify '
                            'driver information for newly enrolled nodes.')
                        ),
            cfg.StrOpt('discovery_default_driver',
                       default='ipmi',
                       help=_(
                           'The default driver or hardware type to use for '
                           'newly discovered nodes (requires '
                           'enable_node_discovery set to True). It is '
                           'automatically added to enabled_hardware_types.')
                       ),
            cfg.BoolOpt('undercloud_debug',
                        default=True,
                        help=_(
                            'Whether to enable the debug log level for '
                            'Undercloud OpenStack services and Container '
                            'Image Prepare step.')
                        ),
            cfg.BoolOpt('undercloud_enable_selinux',
                        default=True,
                        help=_('Enable or disable SELinux during the '
                               'deployment.')),
            cfg.StrOpt('ipa_otp',
                       default='',
                       help=_(
                           'One Time Password to register Undercloud node '
                           'with an IPA server.')
                       ),
            cfg.BoolOpt('ipxe_enabled',
                        default=True,
                        help=_('Whether to use iPXE for deploy and '
                               'inspection.'),
                        deprecated_name='ipxe_deploy',
                        ),
            cfg.IntOpt('scheduler_max_attempts',
                       default=30, min=1,
                       help=_(
                           'Maximum number of attempts the scheduler will '
                           'make when deploying the instance. You should keep '
                           'it greater or equal to the number of bare metal '
                           'nodes you expect to deploy at once to work around '
                           'potential race condition when scheduling.')),
            cfg.BoolOpt('clean_nodes',
                        default=False,
                        help=_(
                            'Whether to clean overcloud nodes (wipe the hard '
                            'drive) between deployments and after the '
                            'introspection.')),
            cfg.ListOpt('enabled_hardware_types',
                        default=['ipmi', 'redfish', 'ilo', 'idrac'],
                        help=_('List of enabled bare metal hardware types '
                               '(next generation drivers).')),
            cfg.BoolOpt('enable_routed_networks',
                        default=False,
                        help=_(
                            'Enable support for routed ctlplane networks.')),
            cfg.BoolOpt('enable_swift_encryption',
                        default=False,
                        help=_(
                            'Whether to enable Swift encryption at-rest or '
                            'not.'
                        )),
            cfg.ListOpt('additional_architectures',
                        default=[],
                        help=(_(
                              'List of additional architectures enabled in '
                              'your cloud environment. The list of supported '
                              'values is: %s') %
                              ' '.join(constants.ADDITIONAL_ARCHITECTURES))
                        ),
            cfg.StrOpt('ipv6_address_mode',
                       default='dhcpv6-stateless',
                       choices=[
                           ('dhcpv6-stateless', 'Address configuration using '
                                                'RA and optional information '
                                                'using DHCPv6.'),
                           ('dhcpv6-stateful', 'Address configuration and '
                                               'optional information using '
                                               'DHCPv6.')
                       ],
                       help=(_('IPv6 address configuration mode for the '
                               'undercloud provisioning network.'))
                       ),
            cfg.ListOpt('ironic_enabled_network_interfaces',
                        default=['flat'],
                        help=(_('Enabled ironic network interface '
                                'implementations. Each hardware type must '
                                'have at least one valid implementation '
                                'enabled.'))
                        ),
            cfg.StrOpt('ironic_default_network_interface',
                       default='flat',
                       choices=[
                           ('flat', 'Use one flat provider network.'),
                           ('neutron', 'Ironic interacts with Neutron to '
                                       'enable other network types and '
                                       'advanced networking features.')
                       ],
                       help=(_('Ironic network interface implementation to '
                               'use by default.'))
                       ),
        ]
        return self.sort_opts(_base_opts + _opts)

    def get_opts(self):
        _base_opts = self.get_base_opts()
        _service_opts = self.get_undercloud_service_opts()
        return self.sort_opts(_base_opts + _service_opts)

    def get_local_subnet_opts(self):
        _subnets_opts = [
            cfg.StrOpt('cidr',
                       default=constants.CTLPLANE_CIDR_DEFAULT,
                       deprecated_opts=_deprecated_opt_network_cidr,
                       help=CIDR_HELP_STR),
            cfg.ListOpt('dhcp_start',
                        default=constants.CTLPLANE_DHCP_START_DEFAULT,
                        deprecated_opts=_deprecated_opt_dhcp_start,
                        help=DHCP_START_HELP_STR),
            cfg.ListOpt('dhcp_end',
                        default=constants.CTLPLANE_DHCP_END_DEFAULT,
                        deprecated_opts=_deprecated_opt_dhcp_end,
                        help=DHCP_END_HELP_STR),
            cfg.ListOpt('dhcp_exclude',
                        default=[],
                        help=DHCP_EXCLUDE_HELP_STR),
            cfg.StrOpt('inspection_iprange',
                       default=constants.CTLPLANE_INSPECTION_IPRANGE_DEFAULT,
                       deprecated_opts=_deprecated_opt_inspection_iprange,
                       help=INSPECTION_IPRANGE_HELP_STR),
            cfg.StrOpt('gateway',
                       default=constants.CTLPLANE_GATEWAY_DEFAULT,
                       deprecated_opts=_deprecated_opt_network_gateway,
                       help=GATEWAY_HELP_STR),
            cfg.BoolOpt('masquerade',
                        default=False,
                        help=MASQUERADE_HELP_STR),
            cfg.ListOpt('host_routes',
                        item_type=cfg.types.Dict(bounds=True),
                        bounds=True,
                        default=[],
                        sample_default=('[{destination: 10.10.10.0/24, '
                                        'nexthop: 192.168.24.1}]'),
                        help=HOST_ROUTES_HELP_STR),
            cfg.ListOpt('dns_nameservers',
                        default=constants.CTLPLANE_DNS_NAMESERVERS_DEFAULT,
                        help=DNS_NAMESERVERS_HELP_STR),
        ]
        return self.sort_opts(_subnets_opts)

    def get_remote_subnet_opts(self):
        _subnets_opts = [
            cfg.StrOpt('cidr',
                       help=CIDR_HELP_STR),
            cfg.ListOpt('dhcp_start',
                        default=[],
                        help=DHCP_START_HELP_STR),
            cfg.ListOpt('dhcp_end',
                        default=[],
                        help=DHCP_END_HELP_STR),
            cfg.ListOpt('dhcp_exclude',
                        default=[],
                        help=DHCP_EXCLUDE_HELP_STR),
            cfg.StrOpt('inspection_iprange',
                       help=INSPECTION_IPRANGE_HELP_STR),
            cfg.StrOpt('gateway',
                       help=GATEWAY_HELP_STR),
            cfg.BoolOpt('masquerade',
                        default=False,
                        help=MASQUERADE_HELP_STR),
            cfg.ListOpt('host_routes',
                        item_type=cfg.types.Dict(bounds=True),
                        bounds=True,
                        default=[],
                        help=HOST_ROUTES_HELP_STR),
            cfg.ListOpt('dns_nameservers',
                        default=constants.CTLPLANE_DNS_NAMESERVERS_DEFAULT,
                        help=DNS_NAMESERVERS_HELP_STR),
        ]
        return self.sort_opts(_subnets_opts)


def list_opts():
    """List config opts for oslo config generator"""
    config = UndercloudConfig()
    _opts = config.get_opts()
    return [(None, copy.deepcopy(_opts)),
            (SUBNETS_DEFAULT[0],
             copy.deepcopy(config.get_local_subnet_opts()))]


def load_global_config():
    """Register UndercloudConfig options into global config"""
    _opts = UndercloudConfig().get_opts()
    CONF.register_opts(_opts)



#   Copyright 2018 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

from oslo_config import cfg
from tripleoclient.config.base import BaseConfig
from tripleoclient.tests import base


class TestBaseConfig(base.TestCase):
    def setUp(self):
        super(TestBaseConfig, self).setUp()
        # Get the class object to test
        self.config = BaseConfig()

    def test_sort_opts(self):
        _opts = [
            cfg.BoolOpt('b', default=True),
            cfg.BoolOpt('a', default=True)
        ]
        expected = [
            cfg.BoolOpt('a', default=True),
            cfg.BoolOpt('b', default=True)
        ]
        ret = self.config.sort_opts(_opts)
        self.assertEqual(expected, ret)

    def test_get_base_opts(self):
        ret = self.config.get_base_opts()
        expected = ['cleanup', 'output_dir']
        self.assertEqual(expected, [x.name for x in ret])



#   Copyright 2015 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import mock

from osc_lib.tests.utils import ParserException
from tripleoclient import constants
from tripleoclient import exceptions
from tripleoclient.tests.v1.overcloud_update import fakes
from tripleoclient.v1 import overcloud_update


class TestOvercloudUpdatePrepare(fakes.TestOvercloudUpdatePrepare):

    def setUp(self):
        super(TestOvercloudUpdatePrepare, self).setUp()

        # Get the command object to test
        app_args = mock.Mock()
        app_args.verbose_level = 1
        self.cmd = overcloud_update.UpdatePrepare(self.app, app_args)

        uuid4_patcher = mock.patch('uuid.uuid4', return_value="UUID4")
        self.mock_uuid4 = uuid4_patcher.start()
        self.addCleanup(self.mock_uuid4.stop)

    @mock.patch('tripleoclient.v1.overcloud_deploy.DeployOvercloud.'
                '_get_ctlplane_attrs', autospec=True, return_value={})
    @mock.patch('tripleoclient.utils.ensure_run_as_normal_user')
    @mock.patch('tripleoclient.utils.prompt_user_for_confirmation',
                return_value=True)
    @mock.patch('tripleoclient.v1.overcloud_deploy.DeployOvercloud.'
                '_get_undercloud_host_entry', autospec=True,
                return_value='192.168.0.1 uc.ctlplane.localhost uc.ctlplane')
    @mock.patch('tripleoclient.v1.overcloud_update.UpdatePrepare.log',
                autospec=True)
    @mock.patch('os.path.abspath')
    @mock.patch('yaml.safe_load')
    @mock.patch('shutil.copytree', autospec=True)
    @mock.patch('six.moves.builtins.open')
    @mock.patch('tripleoclient.v1.overcloud_deploy.DeployOvercloud.'
                'take_action', autospec=True)
    def test_update_out(self, mock_deploy, mock_open, mock_copy, mock_yaml,
                        mock_abspath, mock_logger,
                        mock_get_undercloud_host_entry,
                        mock_confirm, mock_usercheck,
                        mock_get_ctlplane_attrs):
        mock_yaml.return_value = {'fake_container': 'fake_value'}

        argslist = ['--stack', 'overcloud', '--templates']

        verifylist = [
            ('stack', 'overcloud'),
            ('templates', constants.TRIPLEO_HEAT_TEMPLATES),
        ]

        parsed_args = self.check_parser(self.cmd, argslist, verifylist)
        with mock.patch('os.path.exists') as mock_exists, \
                mock.patch('os.path.isfile') as mock_isfile:
            mock_exists.return_value = True
            mock_isfile.return_value = True
            self.cmd.take_action(parsed_args)
            mock_usercheck.assert_called_once()
            mock_deploy.assert_called_once()

    @mock.patch('tripleoclient.utils.ensure_run_as_normal_user')
    @mock.patch('tripleoclient.utils.prompt_user_for_confirmation',
                return_value=True)
    @mock.patch('six.moves.builtins.open')
    @mock.patch('os.path.abspath')
    @mock.patch('yaml.safe_load')
    @mock.patch('shutil.copytree', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_deploy.DeployOvercloud.'
                'take_action', autospec=True)
    def test_update_failed(self, mock_deploy, mock_copy, mock_yaml,
                           mock_abspath, mock_open,
                           mock_confirm, mock_usercheck):
        mock_deploy.side_effect = exceptions.DeploymentError()
        mock_yaml.return_value = {'fake_container': 'fake_value'}
        argslist = ['--stack', 'overcloud', '--templates', ]
        verifylist = [
            ('stack', 'overcloud'),
            ('templates', constants.TRIPLEO_HEAT_TEMPLATES),
        ]
        parsed_args = self.check_parser(self.cmd, argslist, verifylist)

        with mock.patch('os.path.exists') as mock_exists, \
                mock.patch('os.path.isfile') as mock_isfile:
            mock_exists.return_value = True
            mock_isfile.return_value = True
            self.assertRaises(exceptions.DeploymentError,
                              self.cmd.take_action, parsed_args)
            mock_usercheck.assert_called_once()


class TestOvercloudUpdateRun(fakes.TestOvercloudUpdateRun):

    def setUp(self):
        super(TestOvercloudUpdateRun, self).setUp()

        # Get the command object to test
        app_args = mock.Mock()
        app_args.verbose_level = 1
        self.cmd = overcloud_update.UpdateRun(self.app, app_args)

        uuid4_patcher = mock.patch('uuid.uuid4', return_value="UUID4")
        self.mock_uuid4 = uuid4_patcher.start()
        self.addCleanup(self.mock_uuid4.stop)

    @mock.patch('tripleoclient.utils.prompt_user_for_confirmation',
                return_value=True)
    @mock.patch('tripleoclient.utils.run_ansible_playbook',
                autospec=True)
    @mock.patch('os.path.expanduser')
    @mock.patch('oslo_concurrency.processutils.execute')
    @mock.patch('six.moves.builtins.open')
    def test_update_with_no_limit(
            self, mock_open, mock_execute, mock_expanduser, update_ansible,
            mock_confirm):
        mock_expanduser.return_value = '/home/fake/'
        argslist = []
        verifylist = [
            ('static_inventory', None),
            ('playbook', 'all')
        ]
        self.assertRaises(ParserException, lambda: self.check_parser(
            self.cmd, argslist, verifylist))


class TestOvercloudUpdateConverge(fakes.TestOvercloudUpdateConverge):

    def setUp(self):
        super(TestOvercloudUpdateConverge, self).setUp()

        # Get the command object to test
        app_args = mock.Mock()
        app_args.verbose_level = 1
        self.cmd = overcloud_update.UpdateConverge(self.app, app_args)

    @mock.patch('tripleoclient.utils.ensure_run_as_normal_user')
    @mock.patch('tripleoclient.utils.prompt_user_for_confirmation',
                return_value=True)
    @mock.patch(
        'tripleoclient.v1.overcloud_deploy.DeployOvercloud.take_action')
    def test_update_converge(self, deploy_action, mock_confirm,
                             mock_usercheck):
        argslist = ['--templates', '--stack', 'cloud']
        verifylist = [
            ('stack', 'cloud')
        ]
        parsed_args = self.check_parser(self.cmd, argslist, verifylist)

        with mock.patch('os.path.exists') as mock_exists, \
                mock.patch('os.path.isfile') as mock_isfile:
            mock_exists.return_value = True
            mock_isfile.return_value = True
            self.cmd.take_action(parsed_args)
            mock_usercheck.assert_called_once()
            assert('/usr/share/openstack-tripleo-heat-templates/'
                   'environments/lifecycle/update-converge.yaml'
                   in parsed_args.environment_files)
            deploy_action.assert_called_once_with(parsed_args)



#   Copyright 2020 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import mock

from osc_lib import exceptions as osc_lib_exc

from tripleoclient.tests import fakes
from tripleoclient.v2 import overcloud_network


class TestOvercloudNetworkExtract(fakes.FakePlaybookExecution):

    def setUp(self):
        super(TestOvercloudNetworkExtract, self).setUp()

        # Get the command object to test
        app_args = mock.Mock()
        app_args.verbose_level = 1
        self.app.options = fakes.FakeOptions()
        self.cmd = overcloud_network.OvercloudNetworkExtract(self.app,
                                                             app_args)

    @mock.patch('tripleoclient.utils.TempDirs', autospect=True)
    @mock.patch('os.path.abspath', autospect=True)
    @mock.patch('tripleoclient.utils.run_ansible_playbook', autospec=True)
    def test_overcloud_network_extract(self, mock_playbook, mock_abspath,
                                       mock_tempdirs):
        mock_abspath.return_value = '/test/test'
        arglist = ['--stack', 'overcloud', '--output', 'test', '--yes']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.cmd.take_action(parsed_args)
        mock_playbook.assert_called_once_with(
            workdir=mock.ANY,
            playbook='cli-overcloud-network-extract.yaml',
            inventory=mock.ANY,
            playbook_dir=mock.ANY,
            verbosity=3,
            extra_vars={
                "stack_name": 'overcloud',
                "output": '/test/test',
                "overwrite": True
            }
        )

    @mock.patch('os.path.abspath', autospect=True)
    @mock.patch('os.path.exists', autospect=True)
    def test_overcloud_network_extract_no_overwrite(self, mock_abspath,
                                                    mock_path_exists):
        mock_abspath.return_value = '/test/test'
        mock_path_exists.return_value = True
        arglist = ['--stack', 'overcloud', '--output', 'test']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.assertRaises(osc_lib_exc.CommandError,
                          self.cmd.take_action, parsed_args)


class TestOvercloudNetworkProvision(fakes.FakePlaybookExecution):

    def setUp(self):
        super(TestOvercloudNetworkProvision, self).setUp()

        # Get the command object to test
        app_args = mock.Mock()
        app_args.verbose_level = 1
        self.app.options = fakes.FakeOptions()
        self.cmd = overcloud_network.OvercloudNetworkProvision(self.app, None)
        self.cmd.app_args = mock.Mock(verbose_level=1)

    @mock.patch('tripleoclient.utils.TempDirs', autospect=True)
    @mock.patch('os.path.abspath', autospect=True)
    @mock.patch('os.path.exists', autospect=True)
    @mock.patch('tripleoclient.utils.run_ansible_playbook', autospec=True)
    def test_overcloud_network_provision(self, mock_playbook, mock_path_exists,
                                         mock_abspath, mock_tempdirs):
        arglist = ['--output', 'deployed_networks.yaml', '--yes',
                   'network_data_v2.yaml']
        parsed_args = self.check_parser(self.cmd, arglist, [])

        mock_abspath.side_effect = ['/test/network_data_v2.yaml',
                                    '/test/deployed_networks.yaml']
        mock_path_exists.side_effect = [True, True]
        self.cmd.take_action(parsed_args)
        mock_playbook.assert_called_once_with(
            workdir=mock.ANY,
            playbook='cli-overcloud-network-provision.yaml',
            inventory=mock.ANY,
            playbook_dir=mock.ANY,
            verbosity=3,
            extra_vars={
                "network_data_path": '/test/network_data_v2.yaml',
                "network_deployed_path": '/test/deployed_networks.yaml',
                "overwrite": True
            }
        )

    @mock.patch('os.path.abspath', autospect=True)
    @mock.patch('os.path.exists', autospect=True)
    def test_overcloud_network_extract_no_overwrite(self, mock_abspath,
                                                    mock_path_exists):
        arglist = ['--output', 'deployed_networks.yaml', 'network-data.yaml']
        parsed_args = self.check_parser(self.cmd, arglist, [])

        mock_abspath.side_effect = ['/test/network_data_v2.yaml',
                                    '/test/deployed_networks.yaml']
        mock_path_exists.side_effect = [True, True]

        self.assertRaises(osc_lib_exc.CommandError,
                          self.cmd.take_action, parsed_args)



#   Copyright 2015 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import logging

from osc_lib.i18n import _

from tripleoclient import command
from tripleoclient import exceptions
from tripleoclient import utils


class MatchProfiles(command.Command):
    """Assign and validate profiles on nodes"""

    log = logging.getLogger(__name__ + ".MatchProfiles")

    def get_parser(self, prog_name):
        parser = super(MatchProfiles, self).get_parser(prog_name)
        parser.add_argument(
            '--dry-run',
            action='store_true',
            default=False,
            help=_('Only run validations, but do not apply any changes.')
        )
        utils.add_deployment_plan_arguments(parser)
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        bm_client = self.app.client_manager.baremetal

        flavors = self._collect_flavors(parsed_args)

        errors, warnings = utils.assign_and_verify_profiles(
            bm_client, flavors,
            assign_profiles=True,
            dry_run=parsed_args.dry_run
        )
        if errors:
            raise exceptions.ProfileMatchingError(
                _('Failed to validate and assign profiles.'))

    def _collect_flavors(self, parsed_args):
        """Collect nova flavors in use.

        :returns: dictionary flavor name -> (flavor object, scale)
        """
        compute_client = self.app.client_manager.compute

        flavors = {f.name: f for f in compute_client.flavors.list()}
        result = {}

        message = "Provided --{}-flavor, '{}', does not exist"

        for target, (flavor_name, scale) in (
            utils.get_roles_info(parsed_args).items()
        ):
            if flavor_name is None or not scale:
                self.log.debug("--{}-flavor not used".format(target))
                continue

            try:
                flavor = flavors[flavor_name]
            except KeyError:
                raise exceptions.ProfileMatchingError(
                    message.format(target, flavor_name))

            result[flavor_name] = (flavor, scale)

        return result


POSTFIX = '_profile'


class ListProfiles(command.Lister):
    """List overcloud node profiles"""

    log = logging.getLogger(__name__ + ".ListProfiles")

    def get_parser(self, prog_name):
        parser = super(ListProfiles, self).get_parser(prog_name)
        parser.add_argument(
            '--all',
            action='store_true',
            default=False,
            help=_('List all nodes, even those not available to Nova.')
        )
        utils.add_deployment_plan_arguments(parser)
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        bm_client = self.app.client_manager.baremetal
        compute_client = self.app.client_manager.compute

        hypervisors = {h.hypervisor_hostname: h
                       for h in compute_client.hypervisors.list()
                       if h.hypervisor_type == 'ironic'}
        result = []

        maintenance = None if parsed_args.all else False
        for node in bm_client.node.list(detail=True, maintenance=maintenance):
            error = ''

            if node.provision_state not in ('active', 'available'):
                error = "Provision state %s" % node.provision_state
            elif node.power_state in (None, 'error'):
                error = "Power state %s" % node.power_state
            elif node.maintenance:
                error = "Maintenance"
            else:
                try:
                    hypervisor = hypervisors[node.uuid]
                except KeyError:
                    error = 'No hypervisor record'
                else:
                    if hypervisor.status != 'enabled':
                        error = 'Compute service disabled'
                    elif hypervisor.state != 'up':
                        error = 'Compute service down'

            if error and not parsed_args.all:
                continue

            caps = utils.node_get_capabilities(node)
            profile = caps.get('profile')
            possible_profiles = [k[:-len(POSTFIX)]
                                 for k, v in caps.items()
                                 if k.endswith(POSTFIX) and
                                 v.lower() in ('1', 'true')]
            # sorting for convenient display and testing
            possible_profiles.sort()

            record = (node.uuid, node.name or '', node.provision_state,
                      profile, ', '.join(possible_profiles))
            if parsed_args.all:
                record += (error,)
            result.append(record)

        cols = ("Node UUID", "Node Name", "Provision State", "Current Profile",
                "Possible Profiles")
        if parsed_args.all:
            cols += ('Error',)
        return (cols, result)



#   Copyright 2020 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import argparse
import logging

from osc_lib.command import command
from osc_lib.i18n import _

from tripleoclient import constants
from tripleoclient import utils

LOG = logging.getLogger(__name__ + ".BackupOvercloud")


class BackupOvercloud(command.Command):
    """Backup the Overcloud"""

    def get_parser(self, prog_name):
        parser = argparse.ArgumentParser(
            description=self.get_description(),
            prog=prog_name,
            add_help=False
        )

        parser.add_argument(
            '--init',
            default=False,
            action='store_true',
            help=_("Initialize enviornment for backup,"
                   "which will check for package install"
                   "status and configured ReaR.")
        )

        parser.add_argument(
            '--inventory',
            default='/home/stack/tripleo-inventory.yaml',
            help=_("Tripleo inventory file generated with"
                   "tripleo-ansible-inventory command.")
        )

        parser.add_argument(
            '--storage-ip',
            help=_("Storage IP is an optional parameter"
                   "which allows for an ip of a storage"
                   "server to be specified, overriding the"
                   "default undercloud.")
        )

        return parser

    def _run_backup_Overcloud(self, parsed_args):
        """Backup defined overcloud nodes."""

        if parsed_args.init is False:
            playbook = 'cli-overcloud-backup.yaml'
            skip_tags = None
        elif parsed_args.init is True:
            playbook = 'prepare-overcloud-backup.yaml'
            skip_tags = 'bar_create_recover_image, bar_setup_nfs_server'

        if parsed_args.storage_ip:
            extra_vars = {
                "tripleo_backup_and_restore_nfs_server": parsed_args.storage_ip
                }
        else:
            extra_vars = None

        LOG.debug(_('Starting Overcloud Backup'))
        with utils.TempDirs() as tmp:
            utils.run_ansible_playbook(
                playbook=playbook,
                inventory=parsed_args.inventory,
                workdir=tmp,
                playbook_dir=constants.ANSIBLE_TRIPLEO_PLAYBOOKS,
                skip_tags=skip_tags,
                verbosity=utils.playbook_verbosity(self=self),
                extra_vars=extra_vars
                )

    def take_action(self, parsed_args):

        self._run_backup_Overcloud(parsed_args)
        print(
            '\n'
            ' #############################################################\n'
            ' #                  Disclaimer                               #\n'
            ' # Backup verification is the End Users responsibility       #\n'
            ' # Please verify backup integrity before any possible        #\n'
            ' # disruptive actions against the Overcloud. The resulting  #\n'
            ' # backup file path will be shown on a successful execution. #\n'
            ' #                                                           #\n'
            ' # .-Stay safe and avoid future issues-.                     #\n'
            ' #############################################################\n'
            )



#   Copyright 2015 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

"""Plugin action implementation"""

import argparse
import logging
import os
import subprocess
import sys

from openstackclient.i18n import _

from oslo_config import cfg

from tripleoclient import command
from tripleoclient import constants
from tripleoclient import exceptions
from tripleoclient import utils
from tripleoclient.v1 import undercloud_config

UNDERCLOUD_FAILURE_MESSAGE = """
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

An error has occured while deploying the Undercloud.

See the previous output for details about what went wrong.

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
"""

UNDERCLOUD_COMPLETION_MESSAGE = """
##########################################################

The Undercloud has been successfully installed.

Useful files:

Password file is at {0}
The stackrc file is at {1}

Use these files to interact with OpenStack services, and
ensure they are secured.

##########################################################
"""
UNDERCLOUD_UPGRADE_COMPLETION_MESSAGE = """
##########################################################

The Undercloud has been successfully upgraded.

Useful files:

Password file is at {0}
The stackrc file is at {1}

Use these files to interact with OpenStack services, and
ensure they are secured.

##########################################################
"""


class InstallUndercloud(command.Command):
    """Install and setup the undercloud"""

    auth_required = False
    log = logging.getLogger(__name__ + ".InstallUndercloud")
    osloconfig = cfg.CONF

    def get_parser(self, prog_name):
        parser = argparse.ArgumentParser(
            description=self.get_description(),
            prog=prog_name,
            add_help=False
        )
        parser.add_argument('--force-stack-update',
                            dest='force_stack_update',
                            action='store_true',
                            default=False,
                            help=_("Do a virtual update of the ephemeral "
                                   "heat stack. New or failed deployments "
                                   "always have the stack_action=CREATE. This "
                                   "option enforces stack_action=UPDATE."),
                            )
        parser.add_argument(
            '--no-validations',
            dest='no_validations',
            action='store_true',
            default=False,
            help=_("Do not perform undercloud configuration validations"),
        )
        parser.add_argument(
            '--inflight-validations',
            dest='inflight',
            action='store_true',
            default=False,
            help=_('Activate in-flight validations during the deploy. '
                   'In-flight validations provide a robust way to ensure '
                   'deployed services are running right after their '
                   'activation. Defaults to False.')
        )
        parser.add_argument(
            '--dry-run',
            dest='dry_run',
            action='store_true',
            default=False,
            help=_("Print the install command instead of running it"),
        )
        parser.add_argument('-y', '--yes', default=False,
                            action='store_true',
                            help=_("Skip yes/no prompt (assume yes)."))
        parser.add_argument(
            '--disable-container-prepare',
            action='store_true',
            default=False,
            help=_('Disable the container preparation actions to prevent '
                   'container tags from being updated and new containers '
                   'from being fetched. If you skip this but do not have '
                   'the container parameters configured, the deployment '
                   'action may fail.')
        )
        return parser

    def take_action(self, parsed_args):
        # Fetch configuration used to add logging to a file
        utils.load_config(self.osloconfig, constants.UNDERCLOUD_CONF_PATH)
        utils.configure_logging(self.log, self.app_args.verbose_level,
                                self.osloconfig['undercloud_log_file'])
        self.log.debug("take_action(%s)" % parsed_args)

        utils.ensure_run_as_normal_user()
        no_validations = parsed_args.dry_run or parsed_args.no_validations
        inflight = not parsed_args.dry_run and parsed_args.inflight

        cmd = undercloud_config.prepare_undercloud_deploy(
            no_validations=no_validations,
            verbose_level=self.app_args.verbose_level,
            force_stack_update=parsed_args.force_stack_update,
            dry_run=parsed_args.dry_run,
            inflight=inflight,
            disable_container_prepare=parsed_args.disable_container_prepare)

        self.log.warning("Running: %s" % ' '.join(cmd))
        if not parsed_args.dry_run:
            try:
                subprocess.check_call(cmd)
                self.log.warning(UNDERCLOUD_COMPLETION_MESSAGE.format(
                    os.path.join(
                        constants.UNDERCLOUD_OUTPUT_DIR,
                        'undercloud-passwords.conf'
                    ),
                    '~/stackrc'
                    ))
            except Exception as e:
                self.log.error(UNDERCLOUD_FAILURE_MESSAGE)
                self.log.error(e)
                raise exceptions.DeploymentError(e)


class UpgradeUndercloud(InstallUndercloud):
    """Upgrade undercloud"""

    auth_required = False
    log = logging.getLogger(__name__ + ".UpgradeUndercloud")
    osloconfig = cfg.CONF

    def get_parser(self, prog_name):
        parser = super(UpgradeUndercloud, self).get_parser(prog_name)
        parser.add_argument('--skip-package-updates',
                            dest='skip_package_updates',
                            action='store_true',
                            default=False,
                            help=_("Flag to skip the package update when "
                                   "performing upgrades and updates"),
                            )
        return parser

    def _update_extra_packages(self, packages=[], dry_run=False):
        """Necessary packages to be updated before undercloud upgrade."""

        if not packages:
            return

        cmd = ['sudo', 'dnf', 'upgrade', '-y'] + packages

        if not dry_run:
            self.log.warning("Updating necessary packages: {}".format(
                             " ".join(packages)))
            output = utils.run_command(cmd, name="Update extra packages")
            self.log.warning("{}".format(output))
        else:
            self.log.warning("Would update necessary packages: {}".format(
                " ".join(cmd)))

    def _invoke_self(self, parsed_args):
        cmd = ['openstack', 'undercloud', 'upgrade', '--skip-package-updates']
        opts = {'force_stack_update': '--force-stack-update',
                'no_validations': '--no-validations',
                'inflight': '--inflight-validations',
                'dry_run': '--dry-run',
                'yes': '--yes'}
        args = vars(parsed_args)
        for k, v in opts.items():
            if args[k]:
                cmd.append(v)
        # handle --debug
        if self.app_args.verbose_level > 1:
            cmd.append('--debug')
        try:
            subprocess.check_call(cmd)
        except Exception as e:
            self.log.error(e)
            raise exceptions.DeploymentError(e)

    def _run_upgrade(self, parsed_args):
        cmd = undercloud_config.\
            prepare_undercloud_deploy(
                upgrade=True,
                yes=parsed_args.yes,
                no_validations=parsed_args.
                no_validations,
                verbose_level=self.app_args.verbose_level,
                force_stack_update=parsed_args.force_stack_update)
        self.log.warning("Running: %s" % ' '.join(cmd))

        if not parsed_args.dry_run:
            try:
                subprocess.check_call(cmd)
                self.log.warning(
                    UNDERCLOUD_UPGRADE_COMPLETION_MESSAGE.format(
                        os.path.join(
                            constants.UNDERCLOUD_OUTPUT_DIR,
                            'undercloud-passwords.conf'
                        ),
                        '~/stackrc'))
            except Exception as e:
                self.log.error(UNDERCLOUD_FAILURE_MESSAGE)
                self.log.error(e)
                raise exceptions.DeploymentError(e)

    def take_action(self, parsed_args):
        # Fetch configuration used to add logging to a file
        utils.load_config(self.osloconfig, constants.UNDERCLOUD_CONF_PATH)
        utils.configure_logging(self.log, self.app_args.verbose_level,
                                self.osloconfig['undercloud_log_file'])
        self.log.debug("take action(%s)" % parsed_args)

        if (not parsed_args.yes
                and not utils.prompt_user_for_confirmation(
                    constants.UPGRADE_PROMPT, self.log)):
            raise exceptions.UndercloudUpgradeNotConfirmed(
                    constants.UPGRADE_NO)

        utils.ensure_run_as_normal_user()

        if not parsed_args.skip_package_updates:
            if ('python3' in sys.executable):
                pyver = '3'
            else:
                pyver = '2'
            client_pkgs = [
                "python{}-tripleoclient".format(pyver),
            ]
            pkgs = client_pkgs + constants.UNDERCLOUD_EXTRA_PACKAGES
            self._update_extra_packages(pkgs, parsed_args.dry_run)
            self._invoke_self(parsed_args)
        else:
            self._run_upgrade(parsed_args)
